#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  keyboard.py
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import wx


def gen_keymap():
	
	keys = (
			"BACK", "TAB", "RETURN", "ESCAPE", "SPACE", "DELETE", "START",
			"LBUTTON", "RBUTTON", "CANCEL", "MBUTTON", "CLEAR", "PAUSE",
			"CAPITAL", "END", "HOME", "LEFT", "UP", "RIGHT",
			"DOWN", "SELECT", "PRINT", "EXECUTE", "SNAPSHOT", "INSERT", "HELP",
			"NUMPAD0", "NUMPAD1", "NUMPAD2", "NUMPAD3", "NUMPAD4", "NUMPAD5",
			"NUMPAD6", "NUMPAD7", "NUMPAD8", "NUMPAD9", "MULTIPLY", "ADD",
			"SEPARATOR", "SUBTRACT", "DECIMAL", "DIVIDE", "F1", "F2", "F3", "F4",
			"F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12", "F13", "F14",
			"F15", "F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23", "F24",
			"NUMLOCK", "SCROLL", "PAGEUP", "PAGEDOWN", "NUMPAD_SPACE",
			"NUMPAD_TAB", "NUMPAD_ENTER", "NUMPAD_F1", "NUMPAD_F2", "NUMPAD_F3",
			"NUMPAD_F4", "NUMPAD_HOME", "NUMPAD_LEFT", "NUMPAD_UP",
			"NUMPAD_RIGHT", "NUMPAD_DOWN", "NUMPAD_PAGEUP",
			"NUMPAD_PAGEDOWN", "NUMPAD_END", "NUMPAD_BEGIN",
			"NUMPAD_INSERT", "NUMPAD_DELETE", "NUMPAD_EQUAL", "NUMPAD_MULTIPLY",
			"NUMPAD_ADD", "NUMPAD_SEPARATOR", "NUMPAD_SUBTRACT", "NUMPAD_DECIMAL",
			"NUMPAD_DIVIDE"
			)
	
	keyMap = {}
	
	for i in keys:
		keyMap[getattr(wx, "WXK_" + i)] = i
	for i in ("SHIFT", "ALT", "CONTROL", "MENU"):
		keyMap[getattr(wx, "WXK_" + i)] = ''
	
	return keyMap


NAVKEYS = {
		wx.WXK_END, wx.WXK_PAGEUP, wx.WXK_PAGEDOWN,
		wx.WXK_LEFT, wx.WXK_RIGHT, wx.WXK_UP, wx.WXK_DOWN,
		}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  panel_listctrl.py
"""
A custom Panel that acts as a ListCtrl for other wx.Panel objects.

An example ListItem exists that provides two StaticText fields and
can be used as the basis for custom list items
"""
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
# generated by wxGlade 0.9.2 on Thu Jan 16 16:34:51 2020
#

# stdlib
import pathlib

# 3rd party
import wx

# this package
from domdf_wxpython_tools.panel_listctrl.css_parser import parse_css, parse_css_file
from domdf_wxpython_tools.panel_listctrl.font_parser import parse_font


# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


class PanelListCtrl(wx.ScrolledWindow):
	def __init__(
			self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL, name=wx.PanelNameStr, left_padding=32):
			
		wx.ScrolledWindow.__init__(self, parent, id, pos=pos, size=size, style=style | wx.TAB_TRAVERSAL, name=name)

		self._items = []
		self.parent = parent
		self.left_padding = left_padding

		self.SetScrollRate(10, 10)
		
		self.sizer_1 = wx.BoxSizer(wx.VERTICAL)
		self.SetSizer(self.sizer_1)
		self.sizer_1.Fit(self)
		
		self.Layout()
		
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self._on_selection_changed)
		self.Bind(wx.EVT_LIST_KEY_DOWN, self._on_key_down)
		
		self.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_LISTBOX))
		
	def _on_selection_changed(self, event):
		"""
		Handler for EVT_LIST_ITEM_SELECTED, triggered by clicking on an Item
		"""
		
		for item in self._items:
			if item == event.GetEventObject():
				item.SelectItem()
			else:
				item.DeselectItem()
		
		event.Skip()

	def _on_key_down(self, event):
		"""
		Handler for EVT_LIST_KEY_DOWN, triggered by pressing key on keyboard when an item is focused
		"""
		
		key_code = event.GetKeyCode()
		index = self.GetItemPosition(event.GetEventObject())

		self.DeselectAll()
		
		if key_code == wx.WXK_UP:
			index = index - 1
		elif key_code == wx.WXK_DOWN:
			index = index + 1
		
		if key_code == wx.WXK_PAGEUP:
			index = index - 5
		elif key_code == wx.WXK_PAGEDOWN:
			index = index + 5
		
		if index >= self.GetItemCount():
			index = self.GetItemCount()-1
		elif index < 0:
			index = 0
		
		self.Select(index)
		
		event.Skip()
		# TODO: Keyboard autocompletion?

	def SetSelection(self, idx):
		"""
		Set the current selection to the item at the given index
		
		:param idx: index of the item to select
		:type idx: int
		"""
		
		item_to_select = self.GetItem(idx)
		
		for item in self._items:
			if item == item_to_select:
				item.SelectItem()
			else:
				item.DeselectItem()
	
	def DeselectAll(self):
		"""
		Deselect all items
		"""
		
		for i in range(self.GetItemCount()):
			self.Select(i, False)
	
	def AcceptsFocus(self):
		return True
	
	def AcceptsFocusFromKeyboard(self):
		return False
	
	def Append(self, panel_list_item):
		"""
		Append a 'PanelListItem' object, or an instance of a custom subclass, to the control.
		"""
		
		self.sizer_1.Add(panel_list_item, 0, wx.EXPAND, wx.TOP, 0)
		self._items.append(panel_list_item)
		self.sizer_1.Fit(self)
		self.Layout()

	def AppendNewItem(self, text_dict, style_data):
		"""
		Append a new 'PanelListItem' object to the control, passing the 'text_dict' and 'style_data'
		parameters to the new object.
		
		:param text_dict:
		:type text_dict:
		:param style_data:
		:type style_data:
		
		:return: The new PanelListItem object that was added to the control
		:rtype:
		"""
		
		item = PanelListItem(self, text_dict, style_data, left_padding=self.left_padding)
		self.Append(item)
		return item
		
	def Clear(self):
		"""
		Removes all items from the control
		"""
		
		for item in self._items:
			self.sizer_1.Hide(item)
			self.sizer_1.Remove(0)
			
			item.Destroy()
			self.Layout()
		
		self._items = []
		
		event = wx.ListEvent(wx.wxEVT_LIST_DELETE_ALL_ITEMS)
		event.SetEventObject(self)
		wx.PostEvent(self, event)
		
		return True
	
	def DeleteItem(self, item):
		"""
		Deletes the specified item from the control.
		
		:param item:
		:type item:
		
		:return: True if the item was removed, False otherwise (usually because the item wasn't in the control)
		:rtype: bool
		"""
		
		for index, widget in enumerate(self._items):
			if widget == item:
				self.sizer_1.Hide(self._items.pop(index))
				self.sizer_1.Remove(index)
				
				item.Destroy()
				
				self.Layout()
				
				event = wx.ListEvent(wx.wxEVT_LIST_DELETE_ITEM)
				event.SetEventObject(self)
				wx.PostEvent(self, event)
				
				return True
		
		return False

	def Focus(self, idx):
		""" Set Focus to the the given item. """
		for index, item in enumerate(self._items):
			if index == idx:
				item.SelectItem()
			else:
				item.DeselectItem()

	def GetColumnCount(self):
		"""
		Returns the number of columns.
		"""
		
		return 1

#
# 	def GetCountPerPage(self):
# 		"""
# 		GetCountPerPage() -> int
#
# 		Gets the number of items that can fit vertically in the visible area
# 		of the list control (list or report view) or the total number of items
# 		in the list control (icon or small icon view).
# 		"""
# 		return 0
#
	
	def GetFirstSelected(self, *args):
		"""
		Returns the first selected item, or -1 when none is selected.
		
		:param args:
		:type args:
		
		:return:
		:rtype:
		"""
		
		for item in self._items:
			if item.IsSelected():
				return item
		
		return -1

	def GetFocusedItem(self):
		"""
		Gets the currently focused item or -1 if none is focused.
		
		:return:
		:rtype:
		"""
		
		for item in self._items:
			if item.IsSelected():
				return item
		
		return -1

	def GetItem(self, itemIdx, *args):
		"""
		GetItem(itemIdx, col=0) -> ListItem

		Gets information about the item. See SetItem() for more information.
		
		:param itemIdx:
		:type itemIdx:
		:param args:
		:type args:
		
		:return:
		:rtype:
		"""
		
		return self._items[itemIdx]
		
	def GetItemBackgroundColour(self, item):
		"""
		GetItemBackgroundColour(item) -> Colour

		Returns the colour for this item.
		"""
		
		return item.GetBackgroundColour()

	def GetItemCount(self):
		"""
		Returns the number of items in the list control.
		
		:return:
		:rtype:
		"""
		
		return len(self._items)

	def GetItemPosition(self, item):
		"""
		GetItemPosition(item) -> Point

		Returns the position of the item, or -1 if it is not found
		"""
		if item in self._items:
			return self._items.index(item)
		
		return -1

	def GetNextSelected(self, item):
		"""
		Returns subsequent selected items, or -1 when no more are selected.
		
		:param item:
		:type item:
		
		:return:
		:rtype:
		"""
		
		index_of_item = self.GetItemPosition(item)
		
		for index, item in enumerate(self._items):
			if index <= index_of_item:
				continue
			if item.IsSelected():
				return item
		
		return -1

	def GetSelectedItemCount(self):
		"""
		GetSelectedItemCount() -> int

		Returns the number of selected items in the list control.
		"""
		selected_item_count = 0
		
		for item in self._items:
			if item.IsSelected():
				selected_item_count += 1
		
		return selected_item_count
#
# 	def HitTest(self, point):
# 		"""
# 		HitTest(point) -> (long, flags)
#
# 		Determines which item (if any) is at the specified point, giving
# 		details in flags.
# 		"""
# 		pass
#
# 	def InsertItem(self, *__args): with multiple overloads
# 		"""
# 		InsertItem(info) -> long
# 		InsertItem(index, label) -> long
# 		InsertItem(index, imageIndex) -> long
# 		InsertItem(index, label, imageIndex) -> long
#
# 		Inserts an item, returning the index of the new item if successful, -1
# 		otherwise.
# 		"""
# 		return 0
		# TODO: Trigger EVT_LIST_INSERT_ITEM
#
	def IsEmpty(self):
		"""
		Returns true if the control doesn't currently contain any items.
		
		:return:
		:rtype:
		"""
		
		return bool(self._items)

	def IsSelected(self, idx):
		"""
		Returns ``True`` if the item is selected.
		
		:param idx:
		:type idx:
		
		:return:
		:rtype:
		"""
		
		return self._items[idx].IsSelected()

	def RefreshItem(self, item):
		"""
		Redraws the given item.
		
		:param item:
		:type item:
		
		:return:
		:rtype:
		"""
		
		item.Layout()
		item.Refresh()

	def RefreshItems(self, itemFrom, itemTo):
		"""
		Redraws the items between itemFrom and itemTo.
		
		:param itemFrom:
		:type itemFrom:
		:param itemTo:
		:type itemTo:
		
		:return:
		:rtype:
		"""
		
		for index in range(itemFrom, itemTo+1):
			self.RefreshItem(self._items[index])
#
# 	def ScrollList(self, dx, dy):
# 		"""
# 		ScrollList(dx, dy) -> bool
#
# 		Scrolls the list control.
# 		"""
# 		return False

	def Select(self, idx, on=1):
		"""
		Selects/deselects an item.
		
		:param idx:
		:type idx:
		:param on:
		:type on:
		
		:return:
		:rtype:
		"""
		
		self._items[idx].SelectItem(on)

# 	def SortItems(self, fnSortCallBack):
# 		"""
# 		SortItems(fnSortCallBack) -> bool
#
# 		Call this function to sort the items in the list control.
# 		"""
# 		return False
#
	@property
	def ColumnCount(self):
		"""
		Returns the number of columns.
		
		:rtype: int
		"""
		return 1

#
# 	CountPerPage = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
# 	"""GetCountPerPage() -> int
#
# Gets the number of items that can fit vertically in the visible area
# of the list control (list or report view) or the total number of items
# in the list control (icon or small icon view)."""
#
#
	@property
	def FocusedItem(self):
		"""
		Gets the currently focused item or -1 if none is focused.
		
		:return:
		:rtype:
		"""
		
		return self.GetFocusedItem()

	
	@property
	def ItemCount(self):
		"""
		Returns the number of items in the list control.
		
		:rtype: int
		"""
		return len(self._items)

# end of class RecentProjectsPanel


class PanelListItem(wx.Panel):
	def __init__(
			self, parent, text_dict, style_data, id=wx.ID_ANY,
			style=0, name=wx.PanelNameStr, left_padding=32
			):
		"""
		
		:param parent: The PanelListCtrl the item is to go into
		:type parent: PanelListCtrl
		:param text_dict:
		:type text_dict:
		:param style_data:
		:type style_data:
		:param id: An identifier for the panel. ID_ANY is taken to mean a default.
		:type id: wx.WindowID
		:param style: The window style. See wx.Panel.
		:type style: int
		:param name: Window name
		:type name: str
		:param left_padding: the spacing to the left of the text in the control
		:type left_padding: int
		"""
		
		self.parent = parent
		
		if not isinstance(text_dict, dict):
			raise TypeError("'text_dict' must be a dict containing 'css class:text' pairs")
		
		if isinstance(style_data, pathlib.Path):
			# Filename provided
			style_data = parse_css_file(style_data)
		if isinstance(style_data, str):
			# Filename or css provided
			try:
				# CSS
				style_data = parse_css(style_data)
			except ValueError:
				# Filename
				style_data = parse_css_file(style_data)
		
		elif not isinstance(style_data, dict):
			raise TypeError("""'style_data' must be either:
	> A string or pathlib.Path object pointing to a css file, or
	> A dictionary containing the style data, or
	> A string containing css properties.""")
		
		self.style_data = style_data
		
		wx.Panel.__init__(self, parent, id, style=style | wx.TAB_TRAVERSAL | wx.WANTS_CHARS, name=name)
		
		self.selected = False
		
		# Bind events
		self.Bind(wx.EVT_LEFT_UP, self.OnClick)
		self.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)
		self.Bind(wx.EVT_MIDDLE_UP, self.OnMiddleClick)
		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

		# for wxMSW
		self.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
		
		# for wxGTK
		self.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
		
		# Background colour settings for panel
		if "li" in self.style_data and "background-color" in self.style_data["li"]:
			self._default_background = self.style_data["li"]["background-color"]
		else:
			self._default_background = wx.SystemSettings.GetColour(wx.SYS_COLOUR_LISTBOX)
		
		if "li::selection" in self.style_data and "background-color" in self.style_data["li::selection"]:
			self._selected_background = self.style_data["li::selection"]["background-color"]
		else:
			self._selected_background = wx.SystemSettings.GetColour(wx.SYS_COLOUR_MENUHILIGHT)
		
		self._items = {}
		self._text_properties = {}
		
		self.outer_sizer = wx.BoxSizer(wx.HORIZONTAL)
		main_grid = wx.FlexGridSizer(len(text_dict), 2, 0, left_padding)
		
		for index, (css_class, text) in enumerate(text_dict.items()):
			widget = wx.StaticText(self, wx.ID_ANY, text, style=wx.ST_ELLIPSIZE_MIDDLE)
			self._items[css_class] = widget
			
			if index == 0:
				main_grid.Add((0, 0), 0, 0, 0)
				main_grid.Add(widget, 0, wx.TOP, 6)
			else:
				main_grid.Add((0, 0), 0, 0, 0)
				main_grid.Add(widget, 0, wx.TOP, 0 if wx.Platform == "__WXGTK__" else 2)
			
			widget.Bind(wx.EVT_LEFT_UP, self.OnClick)
			widget.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
			widget.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)
			widget.Bind(wx.EVT_MIDDLE_UP, self.OnMiddleClick)
			# for wxMSW
			widget.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
			
			# for wxGTK
			widget.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
		
		self.outer_sizer.Add(main_grid, 0, wx.BOTTOM, 4)
		
		self.SetSizer(self.outer_sizer)
		self.outer_sizer.Fit(self)
		self.Layout()
		
		self.Refresh()
	
	def SelectItem(self, select=True):
		self.selected = select
		if select:
			self.SetFocus()
		self.Refresh()
	
	def DeselectItem(self):
		self.selected = False
		self.Refresh()
		event = wx.ListEvent(wx.wxEVT_LIST_ITEM_DESELECTED)
		event.SetEventObject(self)
		wx.PostEvent(self, event)
	
	def OnRightClick(self, event):
		event = wx.ListEvent(wx.wxEVT_LIST_ITEM_RIGHT_CLICK)
		event.SetEventObject(self)
		wx.PostEvent(self, event)
	
	def OnMiddleClick(self, event):
		event = wx.ListEvent(wx.wxEVT_LIST_ITEM_RIGHT_CLICK)
		event.SetEventObject(self)
		wx.PostEvent(self, event)
	
	def OnClick(self, event):
		event = wx.ListEvent(wx.wxEVT_LIST_ITEM_SELECTED)
		event.SetEventObject(self)
		wx.PostEvent(self, event)
		
	def OnDoubleClick(self, event):
		event = wx.ListEvent(wx.wxEVT_LIST_ITEM_ACTIVATED)
		event.SetEventObject(self)
		wx.PostEvent(self, event)

	def OnKeyDown(self, event):
		key_event = event
		event = wx.ListEvent(wx.wxEVT_LIST_KEY_DOWN)
		event.SetEventObject(self)
		event.SetKeyCode(key_event.GetKeyCode())
		wx.PostEvent(self, event)
	
	def IsSelected(self):
		return self.selected
	
	def _refresh_background_colour(self):
		if self.selected:
			wx.Panel.SetBackgroundColour(self, self._selected_background)
		else:
			wx.Panel.SetBackgroundColour(self, self._default_background)
	
	def _refresh_text(self):
		for classname, widget in self._items.items():
			if self.selected:
				class_style_data = self.style_data[f"li p.{classname}::selection"]
			else:
				class_style_data = self.style_data[f"li p.{classname}"]
				
			colour, font_data = parse_font(class_style_data)
			
			widget.SetForegroundColour(colour)
			widget.SetFont(wx.Font(**font_data))
	
	def SetBackgroundColour(self, colour):
		self._default_background = colour
	
	def SetSelectedBackgroundColour(self, colour):
		self._selected_background = colour
	
	def GetBackgroundColour(self):
		return self._default_background
	
	def GetCurrentBackgroundColour(self):
		return wx.Panel.GetBackgroundColour(self)
	
	def Refresh(self, **kwargs):
		self._refresh_background_colour()
		self._refresh_text()
		wx.Panel.Refresh(self)
	
	def GetContents(self):
		return self._items.values()

# end of class RecentProjectItem


#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  imagepanel.py
"""
Multiline wx.TextCtrl that allows tabbing to the next or previous control.
"""
#
#  Copyright 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#  With help from http://wxpython-users.1045709.n5.nabble.com/Text-control-and-TAB-td2306159.html
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import wx


class TabbableTextCtrl(wx.TextCtrl):
	"""
	Multiline wx.TextCtrl that allows tabbing to the next or previous control.
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, value='',
			pos=wx.DefaultPosition, size=wx.DefaultSize, style=0,
			validator=wx.DefaultValidator, name=wx.TextCtrlNameStr):
		"""
		:param parent: Parent window. Should not be None.
		:type parent: wx.Window
		:param id: Control identifier. A value of -1 denotes a default value.
		:type id: wx.WindowID
		:param value: Default text value.
		:type value: str
		:param pos: Text control position.
		:type pos: wx.Point
		:param size:  Text control size.
		:type size: wx.Size
		:param style: Window style. See wx.TextCtrl.
		:type style: int
		:param validator: Window validator.
		:type validator: wx.Validator
		:param name: Window name.
		:type name: str
		"""
		
		wx.TextCtrl.__init__(
				self, parent, id, value=value, pos=pos, size=size,
				style=style | wx.TE_MULTILINE | wx.TE_PROCESS_ENTER,
				validator=validator, name=name)
		
		self.Bind(wx.EVT_CHAR, self.on_char)
	
	@staticmethod
	def on_char(event):
		"""
		Event handler for key being pressed, to allow for navigating between controls with TAB
		"""
		if event.GetKeyCode() == wx.WXK_TAB:
			flags = wx.NavigationKeyEvent.IsForward
			if event.ShiftDown():
				flags = wx.NavigationKeyEvent.IsBackward
			# if event.ControlDown():
			# 	flags = wx.NavigationKeyEvent.WinChange
			event.GetEventObject().GetParent().Navigate(flags)
		else:
			event.Skip()



#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  events.py
"""
Reusable code for simple events

Usage:

>>> from domdf_wxpython_tools import events

>>> # Create the event outside of the class
>>> myEVT = events.SimpleEvent()

>>> # Set the receiver to the class you want.
>>> # This is usually done from within the receiver class
>>> class MyClass(object):
... 	def __init__(self):
... 		myEVT.set_receiver(self)
...
... 		# Then bind the event to a handler
... 		myEVT.Bind(self.handler)
...
... 	def handler(self):
... 		'''Handler for myEVT'''
... 		pass

>>> # From within the thread, trigger the event with the following syntax:

>>> myEVT.trigger()

"""
#
#  Copyright (c) 2019-2020.  Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

from collections import OrderedDict

import wx


class PayloadEvent(wx.PyCommandEvent):
	"""Event containing a message payload"""
	
	def __init__(self, etype, eid, value):
		"""Creates the event object"""
		wx.PyCommandEvent.__init__(self, etype, eid)
		self.value = value
	
	def GetValue(self):
		"""Returns the value from the event.
		@return: the value of this event

		"""
		return self.value


class SimpleEvent(object):
	"""
	SimpleEvent(receiver, name, event, binder)
	"""
	
	_fields = ("receiver", "name", "event", "binder")
	
	def __init__(self, receiver=None, name="Event"):
		"""

		:param receiver:
		:param name:
		"""
		
		self.receiver = receiver
		self.name = name
		self.event = wx.NewEventType()
		self.binder = wx.PyEventBinder(self.event, 1)
		self.value = None
		self.bindings = {}
	
	def __repr__(self):
		"""
		Return a nicely formatted representation string
		"""
		
		return f'SimpleEvent(name={self.name})'
	
	def __dict__(self):
		"""
		Return a new OrderedDict which maps field names to their values

		:return:
		:rtype: OrderedDict
		"""
		
		return OrderedDict(zip(self._fields, self))
	
	def set_receiver(self, receiver):
		"""
		Set the class that is to receive the event trigger

		:param receiver:
		"""
		
		self.receiver = receiver
	
	def Bind(self, handler, receiver=None, **kwargs):
		"""
		Bind the event to the handler

		:param handler: handler to bind the event to
		:param kwargs: keyword arguments to pass through to receiver's Bind method
		"""
		if receiver is None:
			receiver = self.receiver
		
		receiver.Bind(self.binder, handler, **kwargs)
		self.bindings[receiver] = handler
	
	# self.receiver.Bind(self.binder, handler, **kwargs)
	
	def Unbind(self, receiver=None, **kwargs):
		"""
		Unbind the event from the handler

		:param kwargs: keyword arguments to pass through to receiver's Unbind method
		"""
		
		if receiver:
			receiver.Unbind(self.binder, handler=self.bindings[receiver], **kwargs)
		else:
			for receiver, handler in self.bindings.items():
				receiver.Unbind(self.binder, handler=handler, **kwargs)
	
	# self.receiver.Unbind(self.binder, **kwargs)
	
	def trigger(self, value=None):
		"""
		Trigger the event
		"""
		
		if value is not None:
			self.value = value
		
		for receiver in self.bindings:
			wx.PostEvent(receiver, PayloadEvent(self.event, -1, self.value))
		
		# wx.PostEvent(self.receiver, PayloadEvent(self.event, -1, self.value))
		self.value = None





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  projections.py
#
#  Copyright 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Based on https://stackoverflow.com/a/16709952/3092681
#  Copyright 2013 simonb
#  Licensed under CC-BY-SA
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import matplotlib
from matplotlib import axes


class XPanAxes(matplotlib.axes.Axes):
	"""Constrain pan to x-axis"""
	
	name = "XPanAxes"
	
	def drag_pan(self, button, key, x, y):
		# pretend key=='x'
		matplotlib.axes.Axes.drag_pan(self, button, 'x', x, y)


class XPanAxes_NoZoom(matplotlib.axes.Axes):
	"""Constrain pan to x-axis and prevent zooming"""
	
	name = "XPanAxes_NoZoom"
	
	def drag_pan(self, button, key, x, y):
		# pretend key=='x'
		if button != 1:
			return
		matplotlib.axes.Axes.drag_pan(self, button, 'x', x, y)


class NoZoom(matplotlib.axes.Axes):
	"""Prevent zooming in pan mode"""
	
	name = "NoZoom"
	
	def drag_pan(self, button, key, x, y):
		# pretend key=='x'
		if button != 1:
			return
		matplotlib.axes.Axes.drag_pan(self, button, key, x, y)

#
# matplotlib.projections.register_projection(NoZoom)
# matplotlib.projections.register_projection(XPanAxes_NoZoom)
# matplotlib.projections.register_projection(XPanAxes)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  mxMSW_WebView.py
#
"""
Set the emulation level for wxWidgets WebView purely in Python

Based on https://github.com/wxWidgets/wxWidgets/blob/master/include/wx/msw/webview_ie.h
by Marianne Gagnon
Copyright (c) 2010 Marianne Gagnon, 2011 Steven Lamerton

and also based on https://github.com/wxWidgets/Phoenix/blob/master/etg/webview.py
by Robin Dunn
Copyright (c) 2012-2018 by Total Control Software

Both licensed under the wxWindows Licence

"""

import sys
import winreg
import warnings
import pathlib

"""
Note that the highest emulation level may be used even when the
corresponding browser version is not installed.

Using FORCE options is not recommended, DEFAULT can be used to reset level
to the system default.

The value of the constants were taken from

	https://msdn.microsoft.com/library/ee330730.aspx#browser_emulation

and must not be changed.
"""
wxWEBVIEWIE_EMU_DEFAULT = 0,
wxWEBVIEWIE_EMU_IE7 = 7000,
wxWEBVIEWIE_EMU_IE8 = 8000,
wxWEBVIEWIE_EMU_IE8_FORCE = 8888,
wxWEBVIEWIE_EMU_IE9 = 9000,
wxWEBVIEWIE_EMU_IE9_FORCE = 9999,
wxWEBVIEWIE_EMU_IE10 = 10000,
wxWEBVIEWIE_EMU_IE10_FORCE = 10001,
wxWEBVIEWIE_EMU_IE11 = 11000,
wxWEBVIEWIE_EMU_IE11_FORCE = 11001


def MSWSetEmulationLevel(level=wxWEBVIEWIE_EMU_DEFAULT, program_name=None):
	"""
	
	:param level: The emulation level to use, one of the constants specified above
	:type level: int
	:param program_name: The name of the program to set the emulation level for. Defaults to the Python executable
	:type program_name: str or pathlib.Path object
	
	:return: Whether the operation completed successfully
	:rtype: bool
	"""
	
	if not program_name:
		program_name = sys.executable
	
	if not isinstance(program_name, pathlib.Path):
		program_name = pathlib.Path(program_name)
	
	# Registry key where emulation level for programs are set
	IE_EMULATION_KEY = "SOFTWARE\\Microsoft\\Internet Explorer\\Main\\FeatureControl\\FEATURE_BROWSER_EMULATION"
	
	try:
		key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, IE_EMULATION_KEY, access=winreg.KEY_ALL_ACCESS)
	except OSError:
		warnings.warn("Failed to find web view emulation level in the registry")
		return False
	
	if level != wxWEBVIEWIE_EMU_DEFAULT:
		
		winreg.SetValueEx(key, program_name, 0, 4, level)
		winreg.SetValueEx(key, str(program_name.name), 0, 4, level)
	
	else:
		try:
			winreg.DeleteValue(key, program_name)
		except OSError:
			pass
		
		try:
			winreg.DeleteValue(key, str(program_name.name))
		except OSError:
			pass
	
	return True



#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  editable_listbox.py
"""
A Python implementation of wx.EditableListBox, a ListBox with editable items.

Available in two flavours:
	> Vanilla, that accepts any string value; and
	> Numerical, that only accepts numerical values.
	  Those could be str, int, float or decimal.Decimal, but decimal.Decimal
	  is used internally and is the type that will be returned.
"""
#
#  Copyright (c) 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Based on src/generic/editlbox.cpp from wxWidgets
#  Copyright (c) Vaclav Slavik
#  Licenced under the wxWindows licence
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# stdlib
from decimal import Decimal

# 3rd party
import wx
import wx.adv
from mathematical.utils import rounders

# This package
from domdf_wxpython_tools.validators import FloatValidator


# IDs
ID_ELB_DELETE = wx.NewIdRef()
ID_ELB_EDIT = wx.NewIdRef()
ID_ELB_NEW = wx.NewIdRef()
ID_ELB_UP = wx.NewIdRef()
ID_ELB_DOWN = wx.NewIdRef()
ID_ELB_LISTCTRL = wx.NewIdRef()


# XPM for Buttons

edit_btn_xpm = [
	b"16 16 3 1",
	b"   c None",
	b".  c #000000",
	b"+  c #00007F",
	b"                ",
	b"                ",
	b"      .. ..     ",
	b"        .       ",
	b"        .       ",
	b"  ++++  .  ++++ ",
	b"     ++ . ++  ++",
	b"  +++++ . ++++++",
	b" ++  ++ . ++    ",
	b" ++  ++ . ++  ++",
	b"  +++++ .  ++++ ",
	b"        .       ",
	b"        .       ",
	b"      .. ..     ",
	b"                ",
	b"                "
		]

numerical_edit_btn_xpm = [
	b"16 16 3 1",
	b"   c None",
	b".  c #000000",
	b"+  c #00007F",
	b"                ",
	b"                ",
	b"      .. ..     ",
	b"        .       ",
	b"   ++   .  ++++ ",
	b"  +++   . ++  ++",
	b" ++++   . ++  ++",
	b"   ++   . ++  ++",
	b"   ++   . ++  ++",
	b"   ++   . ++  ++",
	b" ++++++ .  ++++ ",
	b"        .       ",
	b"        .       ",
	b"      .. ..     ",
	b"                ",
	b"                "
		]

new_btn_xpm = [
	b"16 16 5 1",
	b"   c None",
	b".  c #7F7F7F",
	b"+  c #FFFFFF",
	b"@  c #FFFF00",
	b"#  c #000000",
	b"                ",
	b"                ",
	b" .  .+ .@       ",
	b"  . .@.@# # #   ",
	b"  @.@+....   #  ",
	b" ... @@         ",
	b"  @ . @.     #  ",
	b" .# .@          ",
	b"    .        #  ",
	b"  #             ",
	b"             #  ",
	b"  #             ",
	b"             #  ",
	b"  # # # # # #   ",
	b"                ",
	b"                "
		]

delete_btn_xpm = [
	b"16 16 3 1",
	b"   c None",
	b".  c #7F0000",
	b"+  c #FFFFFF",
	b"                ",
	b"                ",
	b"                ",
	b" ..+        ..+ ",
	b" ....+     ..+  ",
	b"  ....+   ..+   ",
	b"    ...+ .+     ",
	b"     .....+     ",
	b"      ...+      ",
	b"     .....+     ",
	b"    ...+ ..+    ",
	b"   ...+   ..+   ",
	b"  ...+     .+   ",
	b"  ...+      .+  ",
	b"   .         .  ",
	b"                "
		]

down_btn_xpm = [
	b"16 16 2 1",
	b"   c None",
	b".  c #000000",
	b"                ",
	b"                ",
	b"         ...    ",
	b"        ...     ",
	b"       ...      ",
	b"       ...      ",
	b"       ...      ",
	b"       ...      ",
	b"   ...........  ",
	b"    .........   ",
	b"     .......    ",
	b"      .....     ",
	b"       ...      ",
	b"        .       ",
	b"                ",
	b"                "
		]

up_btn_xpm = [
	b"16 16 2 1",
	b"   c None",
	b".  c #000000",
	b"                ",
	b"        .       ",
	b"       ...      ",
	b"      .....     ",
	b"     .......    ",
	b"    .........   ",
	b"   ...........  ",
	b"       ...      ",
	b"       ...      ",
	b"       ...      ",
	b"       ...      ",
	b"      ...       ",
	b"     ...        ",
	b"                ",
	b"                ",
	b"                "
		]


class CleverListCtrl(wx.ListCtrl):
	"""
	list control with auto-resizable column:
	"""
	def __init__(
			self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition,
			size=wx.DefaultSize, style=wx.LC_ICON,
			validator=wx.DefaultValidator, name=wx.ListCtrlNameStr
			):
		"""
		:param parent:
		:type parent: wx.Window
		:param id:
		:type id: wx.WindowID
		:param pos:
		:type pos: wx.Point
		:param size:
		:type size: wx.Size
		:param style:
		:type style: int
		:param validator:
		:type validator: wx.Validator
		:param name:
		:type name: str
		"""
		
		wx.ListCtrl.__init__(self, parent, id, pos, size, style, validator, name)
		
		self.CreateColumns()
		
		self.Bind(wx.EVT_SIZE, self.OnSize)
	
	def CreateColumns(self):
		self.InsertColumn(0, "item")
		self.SizeColumns()
	
	def SizeColumns(self):
		w = self.GetSize().x
		
		if wx.Platform == "__WXMSW__":
			w -= wx.SystemSettings.GetMetric(wx.SYS_VSCROLL_X, self) + 6
		else:
			w -= 2 * wx.SystemSettings.GetMetric(wx.SYS_VSCROLL_X, self)
		
		if w < 0:
			w = 0
		self.SetColumnWidth(0, w)
	
	# private
	def OnSize(self, event):
		self.SizeColumns()
		event.Skip()


class EditableListBox(wx.Panel):
	def __init__(self, parent, id=wx.ID_ANY, label="",
			pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.adv.EL_DEFAULT_STYLE, name=wx.adv.EditableListBoxNameStr
			):
		"""
		This class provides a composite control that lets the user easily enter
		and edit a list of strings.

		Styles supported:
			> wx.adv.EL_ALLOW_NEW - Allow user to create new items.
			> wx.adv.EL_ALLOW_EDIT - Allow user to edit text in the control.
			> wx.adv.EL_ALLOW_DELETE - Allow user to delete text from the control.

		:param parent:
		:type parent: wx.Window
		:param id:
		:type id: wx.WindowID
		:param label:
		:type label: wx.String
		:param pos:
		:type pos: wx.Point
		:param size:
		:type size:  wx.Size
		:param style:
		:type style: int
		:param name:
		:type name: str
		"""
		
		wx.Panel.__init__(self, parent, id, pos, size, style, name)
		
		self.m_style = style
		
		sizer = wx.BoxSizer(wx.VERTICAL)
		
		subp = wx.Panel(self, style=wx.SUNKEN_BORDER | wx.TAB_TRAVERSAL)
		subsizer = wx.BoxSizer(wx.HORIZONTAL)
		
		subsizer.Add(wx.StaticText(subp, wx.ID_ANY, label),
					 1, wx.CENTER | wx.LEFT, 5)
		# TODO: if width too small put label on row above buttons
		# Min width to see all buttons = (152, 250)
		
		if self.m_style & wx.adv.EL_ALLOW_EDIT:
			self.m_bEdit = wx.BitmapButton(subp, ID_ELB_EDIT, wx.Bitmap(
				edit_btn_xpm))  # wx.ArtProvider.GetBitmap(wx.ART_EDIT, wx.ART_BUTTON))
			self.m_bEdit.SetToolTip("Edit item")
			subsizer.Add(self.m_bEdit, 0, wx.CENTER)
		
		if self.m_style & wx.adv.EL_ALLOW_NEW:
			self.m_bNew = wx.BitmapButton(subp, ID_ELB_NEW,
										  wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_BUTTON))
			self.m_bNew.SetToolTip("New item")
			subsizer.Add(self.m_bNew, 0, wx.CENTER)
		
		if self.m_style & wx.adv.EL_ALLOW_DELETE:
			self.m_bDel = wx.BitmapButton(subp, ID_ELB_DELETE, wx.Bitmap(delete_btn_xpm))
			self.m_bDel.SetToolTip("Delete item")
			subsizer.Add(self.m_bDel, 0, wx.CENTER)
		
		if not (self.m_style & wx.adv.EL_NO_REORDER):
			self.m_bUp = wx.BitmapButton(subp, ID_ELB_UP,
										 wx.ArtProvider.GetBitmap(wx.ART_GO_UP, wx.ART_BUTTON))
			self.m_bUp.SetToolTip("Move up")
			subsizer.Add(self.m_bUp, 0, wx.CENTER)
			
			self.m_bDown = wx.BitmapButton(subp, ID_ELB_DOWN,
										   wx.ArtProvider.GetBitmap(wx.ART_GO_DOWN, wx.ART_BUTTON))
			self.m_bDown.SetToolTip("Move down")
			subsizer.Add(self.m_bDown, 0, wx.CENTER)
		
		subp.SetSizer(subsizer)
		subsizer.Fit(subp)
		
		sizer.Add(subp, 0, wx.EXPAND)
		
		st = wx.LC_REPORT | wx.LC_NO_HEADER | wx.LC_SINGLE_SEL | wx.SUNKEN_BORDER
		
		if style & wx.adv.EL_ALLOW_EDIT:
			st |= wx.LC_EDIT_LABELS
		
		self.m_listCtrl = CleverListCtrl(self, ID_ELB_LISTCTRL, wx.DefaultPosition, wx.DefaultSize, st)
		
		self.SetStrings([])
		
		sizer.Add(self.m_listCtrl, 1, wx.EXPAND)
		
		self.SetSizer(sizer)
		self.Layout()
		
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, id=ID_ELB_LISTCTRL)
		self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginLabelEdit, id=ID_ELB_LISTCTRL)
		self.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnEndLabelEdit, id=ID_ELB_LISTCTRL)
		self.Bind(wx.EVT_BUTTON, self.OnNewItem, id=ID_ELB_NEW)
		self.Bind(wx.EVT_BUTTON, self.OnUpItem, id=ID_ELB_UP)
		self.Bind(wx.EVT_BUTTON, self.OnDownItem, id=ID_ELB_DOWN)
		self.Bind(wx.EVT_BUTTON, self.OnEditItem, id=ID_ELB_EDIT)
		self.Bind(wx.EVT_BUTTON, self.OnDelItem, id=ID_ELB_DELETE)
		self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated, self.m_listCtrl)
	
	def OnItemActivated(self, evt):
		self.curRow = evt.GetIndex()
		print(123)
		self.m_listCtrl.EditLabel(self.curRow)
		evt.Skip()
	
	def SetStrings(self, strings):
		"""
		Replaces current contents with given strings.
		
		:param strings: list of strings
		:type strings: list of str
		"""
		
		self.m_listCtrl.DeleteAllItems()
		for i in range(len(strings)):
			self.m_listCtrl.InsertItem(i, str(strings[i]))
		
		self.m_listCtrl.InsertItem(len(strings), '')
		self.m_listCtrl.SetItemState(0, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
	
	def GetStrings(self):
		"""
		Returns a list of the current contents of the control.
		
		:return: list of strings
		:rtype: list of str
		"""
		
		strings = []
		
		for i in range(self.m_listCtrl.GetItemCount()):
			val = self.m_listCtrl.GetItemText(i)
			if val:
				strings.append(val)
	
		return strings
	
	def OnItemSelected(self, event):
		self.m_selection = event.GetIndex()
		if not (self.m_style & wx.adv.EL_NO_REORDER):
			self.m_bUp.Enable(self.m_selection != 0 and self.m_selection < self.m_listCtrl.GetItemCount() - 1)
			self.m_bDown.Enable(self.m_selection < self.m_listCtrl.GetItemCount() - 2)
			
		if self.m_style & wx.adv.EL_ALLOW_EDIT:
			self.m_bEdit.Enable(self.m_selection < self.m_listCtrl.GetItemCount())
			if self.m_style & wx.adv.EL_ALLOW_DELETE:
				self.m_bDel.Enable(self.m_selection < self.m_listCtrl.GetItemCount() - 1)
	
	def OnNewItem(self, event):
		self.m_listCtrl.SetItemState(
				self.m_listCtrl.GetItemCount() - 1,
				wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED
				)
		
		self.m_listCtrl.EditLabel(self.m_selection)
	
	def OnBeginLabelEdit(self, event):
		event.Skip()
		
		# To ensure it happens after editing starts
		wx.CallAfter(self.SetupEditControl)
	
	def SetupEditControl(self):
		edit_control = self.m_listCtrl.GetEditControl()
		
		# Bind Events
		edit_control.Bind(wx.EVT_TEXT, self.on_value_changed)
		edit_control.Bind(wx.EVT_TEXT_ENTER, self.on_value_changed)
		
		return edit_control
	
	def on_value_changed(self, event):  # wxGlade: CalibreMeasurementPanel.<event_handler>
		event.Skip()
	
	def OnEndLabelEdit(self, event):
		# return
		if event.GetIndex() == self.m_listCtrl.GetItemCount()-1 and event.GetText():
			# The user edited last (empty) line, i.e. added new entry. We have to
			# add new empty line here so that adding one more line is still
			# possible:
			self.m_listCtrl.InsertItem(self.m_listCtrl.GetItemCount(), '')
			
			# Simulate a wxEVT_LIST_ITEM_SELECTED event for the new item,
			# so that the buttons are enabled/disabled properly
			selectionEvent = wx.ListEvent(wx.wxEVT_LIST_ITEM_SELECTED, self.m_listCtrl.GetId())
			selectionEvent.SetIndex(event.GetIndex())
			self.m_listCtrl.GetEventHandler().ProcessEvent(selectionEvent)
	
	def OnDelItem(self, event):
		
		self.m_listCtrl.DeleteItem(self.m_selection)
		self.m_listCtrl.SetItemState(self.m_selection, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
	
	def OnEditItem(self, event):
		
		self.m_listCtrl.EditLabel(self.m_selection)
	
	def SwapItems(self, i1, i2):
		"""
		
		:param i1:
		:type i1: int
		:param i2:
		:type i2: int
		"""
		
		# swap the text
		t1 = self.m_listCtrl.GetItemText(i1)
		t2 = self.m_listCtrl.GetItemText(i2)
		self.m_listCtrl.SetItemText(i1, t2)
		self.m_listCtrl.SetItemText(i2, t1)

		# swap the item data
		d1 = self.m_listCtrl.GetItemData(i1)
		d2 = self.m_listCtrl.GetItemData(i2)
		self.m_listCtrl.SetItemData(i1, d2)
		self.m_listCtrl.SetItemData(i2, d1)
	
	def OnUpItem(self, event):
		
		self.SwapItems(self.m_selection - 1, self.m_selection)
		self.m_listCtrl.SetItemState(
				self.m_selection - 1,
				wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
	
	def OnDownItem(self, event):
		
		self.SwapItems(self.m_selection + 1, self.m_selection)
		self.m_listCtrl.SetItemState(
				self.m_selection + 1,
				wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
	
	def GetListCtrl(self):
		"""
		Returns a reference to the actual list control portion of the custom control.
		
		:return:
		:rtype: wx.ListCtrl
		"""
		
		return self.m_listCtrl
	
	def GetDelButton(self):
		"""
		Retrieves a reference to the BitmapButton that is used as the 'delete' button in the control.
		"""
		
		return self.m_bDel
	
	def GetNewButton(self):
		"""
		Retrieves a reference to the BitmapButton that is used as the 'new' button in the control.
		"""
		
		return self.m_bNew
	
	def GetUpButton(self):
		"""
		Retrieves a reference to the BitmapButton that is used as the 'up' button in the control.
		"""
		
		return self.m_bUp
	
	def GetDownButton(self):
		"""
		Retrieves a reference to the BitmapButton that is used as the 'down' button in the control.
		"""
		
		return self.m_bDown
	
	def GetEditButton(self):
		"""
		Retrieves a reference to the BitmapButton that is used as the 'edit' button in the control.
		"""
		
		return self.m_bEdit
			
			
class EditableNumericalListBox(EditableListBox):
	def __init__(self, parent, id=wx.ID_ANY, label="", decimal_places=-1,
			pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.adv.EL_DEFAULT_STYLE, name=wx.adv.EditableListBoxNameStr
			):
		"""
		
		:param parent:
		:type parent: wx.Window
		:param id:
		:type id: wx.WindowID
		:param label:
		:type label: wx.String
		:param pos:
		:type pos: wx.Point
		:param size:
		:type size:  wx.Size
		:param style:
		:type style: int
		:param name:
		:type name: str
		"""
		
		EditableListBox.__init__(self, parent, id, label, pos, size, style, name)
		
		self.SetDecimalPlaces(decimal_places)
		
		# Change bitmap for Edit button
		self.m_bEdit.SetBitmap(wx.Bitmap(numerical_edit_btn_xpm))
		self.m_bEdit.SetBitmapDisabled(wx.NullBitmap)
	
	def SetDecimalPlaces(self, _decimal_places):
		self._decimal_places = _decimal_places
		
		if self._decimal_places == 0:
			self._rounders_string = "0"
		elif self._decimal_places == -1:
			self._rounders_string = ''
		else:
			self._rounders_string = f"0.{'0' * self._decimal_places}"
	
	def GetDecimalPlaces(self):
		return self._decimal_places
	
	@property
	def decimal_places(self):
		return self._decimal_places
	
	@decimal_places.setter
	def decimal_places(self, decimal_places):
		self.SetDecimalPlaces(decimal_places)
	
	def SetStrings(self, strings):
		self.SetValues(strings)
	
	def SetValues(self, values):
		"""
		Replaces current contents with given values.
		
		:param values: list of values
		:type values: list of int or list of float or list of decimal.Decimal
		
		:return:
		:rtype:
		"""
		
		self.m_listCtrl.DeleteAllItems()
		for i in range(len(values)):
			self.m_listCtrl.InsertItem(i, str(values[i]))
		
		self.m_listCtrl.InsertItem(len(values), '')
		self.m_listCtrl.SetItemState(0, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
	
	def GetStrings(self):
		return self.GetValues()
	
	def GetValues(self):
		"""
		Returns a list of the current contents of the control.
		
		:return:
		:rtype:
		"""
		
		values = []
		
		for i in range(self.m_listCtrl.GetItemCount()):
			val = self.m_listCtrl.GetItemText(i)
			if val:
				if self.decimal_places == -1:
					# Don't format
					values.append(Decimal(val))
				else:
					values.append(rounders(val, self._rounders_string))
		
		return values
	
	def SetupEditControl(self):
		edit_control = self.m_listCtrl.GetEditControl()
		print(edit_control)
		edit_control.SetValidator(FloatValidator(5))
		
		# Bind Events
		edit_control.Bind(wx.EVT_TEXT, self.on_value_changed)
		edit_control.Bind(wx.EVT_TEXT_ENTER, self.on_value_changed)
	
	def on_value_changed(self, event):  # wxGlade: CalibreMeasurementPanel.<event_handler>
		value = event.GetEventObject().GetValue()
		
		if value == ".":
			event.GetEventObject().ChangeValue("0.")
			wx.CallAfter(event.GetEventObject().SetInsertionPointEnd)
			
		event.Skip()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  style_picker.py
#
#  Copyright 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
# generated by wxGlade 0.9.3 on Tue Apr  9 18:07:58 2019
#

# stdlib
import sys

# 3rd party
import wx

# this package
from domdf_wxpython_tools.StylePickerPanel import StylePickerPanel, default_styles
from domdf_wxpython_tools.ColourPickerPanel import ColourPickerPanel, default_colours


# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


class style_picker(wx.Dialog):
	def __init__(
			self, parent, title="Choose Styles", label="Choose Styles: ",
			selection_choices=None, *args, **kwds
			):
		
		self.title = title
		
		args = (parent,) + args
		# begin wxGlade: style_picker.__init__
		kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_DIALOG_STYLE
		wx.Dialog.__init__(self, *args, **kwds)
		self.StylePickerPanel = StylePickerPanel(self, label=label, selection_choices=selection_choices)
		self.button_panel = wx.Panel(self, wx.ID_ANY)
		self.cancel_btn = wx.Button(self.button_panel, wx.ID_ANY, "Cancel")
		self.apply_btn = wx.Button(self.button_panel, wx.ID_ANY, "Apply")

		self.__set_properties()
		self.__do_layout()

		self.Bind(wx.EVT_BUTTON, self.cancel, self.cancel_btn)
		self.Bind(wx.EVT_BUTTON, self.apply, self.apply_btn)
		# end wxGlade
	
	def __set_properties(self):
		# begin wxGlade: style_picker.__set_properties
		self.SetTitle("Choose Styles")
		# end wxGlade
		self.SetTitle(self.title)
	
	def __do_layout(self):
		# begin wxGlade: style_picker.__do_layout
		parent_sizer = wx.FlexGridSizer(2, 1, 0, 0)
		button_grid = wx.GridSizer(1, 2, 0, 5)
		parent_sizer.Add(self.StylePickerPanel, 1, wx.EXPAND, 0)
		button_grid.Add(self.cancel_btn, 0, wx.ALIGN_CENTER, 0)
		button_grid.Add(self.apply_btn, 0, wx.ALIGN_CENTER, 0)
		self.button_panel.SetSizer(button_grid)
		parent_sizer.Add(self.button_panel, 1, wx.ALIGN_BOTTOM | wx.ALIGN_RIGHT | wx.ALL, 5)
		self.SetSizer(parent_sizer)
		parent_sizer.Fit(self)
		self.Layout()
		# end wxGlade

	def cancel(self, _):  # wxGlade: style_picker.<event_handler>
		self.Destroy()
	
	def apply(self, event):  # wxGlade: style_picker.<event_handler>
		self.style_list = self.StylePickerPanel.get_selection()
		event.Skip()
		self.EndModal(wx.ID_OK)

# end of class style_picker


class colour_picker(style_picker):
	def __init__(
			self, parent, title="Choose Colours", label="Choose Colours: ",
			picker_choices=None, selection_choices=None, *args, **kwds
			):
		
		self.title = title
		self.picker_choices = picker_choices

		args = (parent,) + args

		kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_DIALOG_STYLE
		wx.Dialog.__init__(self, *args, **kwds)
		self.StylePickerPanel = ColourPickerPanel(self, label=label, picker_choices=picker_choices, selection_choices=selection_choices)
		self.button_panel = wx.Panel(self, wx.ID_ANY)
		self.cancel_btn = wx.Button(self.button_panel, wx.ID_ANY, "Cancel")
		self.apply_btn = wx.Button(self.button_panel, wx.ID_ANY, "Apply")

		self.__set_properties()
		self.__do_layout()

		self.Bind(wx.EVT_BUTTON, self.cancel, self.cancel_btn)
		self.Bind(wx.EVT_BUTTON, self.apply, self.apply_btn)

	
	def apply(self, event):
		self.colour_list = self.StylePickerPanel.get_selection()
		event.Skip()
		self.EndModal(wx.ID_OK)
	
	def __set_properties(self):
		# begin wxGlade: style_picker.__set_properties
		self.SetTitle("Choose Styles")
		# end wxGlade
		self.SetTitle(self.title)
	
	def __do_layout(self):
		# begin wxGlade: style_picker.__do_layout
		parent_sizer = wx.FlexGridSizer(2, 1, 0, 0)
		button_grid = wx.GridSizer(1, 2, 0, 5)
		parent_sizer.Add(self.StylePickerPanel, 1, wx.EXPAND, 0)
		button_grid.Add(self.cancel_btn, 0, wx.ALIGN_CENTER, 0)
		button_grid.Add(self.apply_btn, 0, wx.ALIGN_CENTER, 0)
		self.button_panel.SetSizer(button_grid)
		parent_sizer.Add(self.button_panel, 1, wx.ALIGN_BOTTOM | wx.ALIGN_RIGHT | wx.ALL, 5)
		self.SetSizer(parent_sizer)
		parent_sizer.Fit(self)
		self.Layout()
		# end wxGlade



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  constants.py
"""
Constants for panel_listctrl
"""
#
#  Copyright (c) 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

# 3rd party
import webcolors
import wx

# Default text settings
text_defaults = {
		"color": webcolors.name_to_hex("black"),
		"font-size": "11pt",
		"font-family": "default",
		"font-style": "normal",
		"font-weight": "normal",
		"text-decoration": "none",
		"font-face-name": ''
		}

if wx.Platform == "__WXMSW__":
	text_defaults["font-size"] = "9pt"
elif wx.Platform == "__WXGTK__":
	text_defaults["font-size"] = "11pt"
	import lsb_release
	
	if "Ubuntu" in lsb_release.get_distro_information().values():
		text_defaults["font-face-name"] = "Ubuntu"

# System colours
sys_colour_lookup = {
		"SYS_COLOUR_SCROLLBAR": wx.SYS_COLOUR_SCROLLBAR,
		"SYS_COLOUR_DESKTOP": wx.SYS_COLOUR_DESKTOP,
		"SYS_COLOUR_BACKGROUND": wx.SYS_COLOUR_DESKTOP,
		"SYS_COLOUR_ACTIVECAPTION": wx.SYS_COLOUR_ACTIVECAPTION,
		"SYS_COLOUR_INACTIVECAPTION": wx.SYS_COLOUR_INACTIVECAPTION,
		"SYS_COLOUR_MENU": wx.SYS_COLOUR_MENU,
		"SYS_COLOUR_WINDOW": wx.SYS_COLOUR_WINDOW,
		"SYS_COLOUR_WINDOWFRAME": wx.SYS_COLOUR_WINDOWFRAME,
		"SYS_COLOUR_MENUTEXT": wx.SYS_COLOUR_MENUTEXT,
		"SYS_COLOUR_WINDOWTEXT": wx.SYS_COLOUR_WINDOWTEXT,
		"SYS_COLOUR_CAPTIONTEXT": wx.SYS_COLOUR_CAPTIONTEXT,
		"SYS_COLOUR_ACTIVEBORDER": wx.SYS_COLOUR_ACTIVEBORDER,
		"SYS_COLOUR_INACTIVEBORDER": wx.SYS_COLOUR_INACTIVEBORDER,
		"SYS_COLOUR_APPWORKSPACE": wx.SYS_COLOUR_APPWORKSPACE,
		"SYS_COLOUR_HIGHLIGHT": wx.SYS_COLOUR_HIGHLIGHT,
		"SYS_COLOUR_HIGHLIGHTTEXT": wx.SYS_COLOUR_HIGHLIGHTTEXT,
		"SYS_COLOUR_BTNFACE": wx.SYS_COLOUR_BTNFACE,
		"SYS_COLOUR_3DFACE": wx.SYS_COLOUR_BTNFACE,
		"SYS_COLOUR_FRAMEBK": wx.SYS_COLOUR_BTNFACE,
		"SYS_COLOUR_BTNSHADOW": wx.SYS_COLOUR_BTNSHADOW,
		"SYS_COLOUR_3DSHADOW": wx.SYS_COLOUR_BTNSHADOW,
		"SYS_COLOUR_GRAYTEXT": wx.SYS_COLOUR_GRAYTEXT,
		"SYS_COLOUR_BTNTEXT": wx.SYS_COLOUR_BTNTEXT,
		"SYS_COLOUR_INACTIVECAPTIONTEXT": wx.SYS_COLOUR_INACTIVECAPTIONTEXT,
		"SYS_COLOUR_BTNHIGHLIGHT": wx.SYS_COLOUR_BTNHIGHLIGHT,
		"SYS_COLOUR_BTNHILIGHT": wx.SYS_COLOUR_BTNHIGHLIGHT,
		"SYS_COLOUR_3DHIGHLIGHT": wx.SYS_COLOUR_BTNHIGHLIGHT,
		"SYS_COLOUR_3DHILIGHT": wx.SYS_COLOUR_BTNHIGHLIGHT,
		"SYS_COLOUR_3DDKSHADOW": wx.SYS_COLOUR_3DDKSHADOW,
		"SYS_COLOUR_3DLIGHT": wx.SYS_COLOUR_3DLIGHT,
		"SYS_COLOUR_INFOTEXT": wx.SYS_COLOUR_INFOTEXT,
		"SYS_COLOUR_INFOBK": wx.SYS_COLOUR_INFOBK,
		"SYS_COLOUR_LISTBOX": wx.SYS_COLOUR_LISTBOX,
		"SYS_COLOUR_HOTLIGHT": wx.SYS_COLOUR_HOTLIGHT,
		"SYS_COLOUR_GRADIENTACTIVECAPTION": wx.SYS_COLOUR_GRADIENTACTIVECAPTION,
		"SYS_COLOUR_GRADIENTINACTIVECAPTION": wx.SYS_COLOUR_GRADIENTINACTIVECAPTION,
		"SYS_COLOUR_MENUHILIGHT": wx.SYS_COLOUR_MENUHILIGHT,
		"SYS_COLOUR_MENUBAR": wx.SYS_COLOUR_MENUBAR,
		"SYS_COLOUR_LISTBOXTEXT": wx.SYS_COLOUR_LISTBOXTEXT,
		"SYS_COLOUR_LISTBOXHIGHLIGHTTEXT": wx.SYS_COLOUR_LISTBOXHIGHLIGHTTEXT,
		}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  css_parser.py
#
#  Copyright (c) 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
# generated by wxGlade 0.9.2 on Thu Jan 16 16:34:51 2020
#

# 3rd party
import tinycss
import webcolors
import wx

# this package
from domdf_wxpython_tools.panel_listctrl.constants import text_defaults, sys_colour_lookup


# Setup tinycss
parser = tinycss.make_parser("page3")


def parse_css_file(filename):
	"""
	Parse the stylesheet in the given file
	
	:param filename: The filename of the stylesheet to parse
	:type filename: str or pathlib.Path
	
	:return: Parsed CSS stylesheet
	:rtype: dict
	"""
	
	stylesheet = parser.parse_stylesheet_file(css_file=str(filename))
	
	return _parse_css(stylesheet)


def parse_css(css_data):
	"""
	Parse the stylesheet from the given string

	:param css_data: A string representing a CSS stylesheel
	:type css_data: str

	:return: Parsed CSS stylesheet
	:rtype: dict
	"""
	
	stylesheet = parser.parse_stylesheet(css_unicode=css_data)
	
	return _parse_css(stylesheet)


def _parse_css(stylesheet):
	"""
	Internal function for actual parsing of css
	
	:param stylesheet: A tinycss parsed stylesheel
	:type stylesheet: :class:`tinycss.css21.Stylesheet`
	:return: Parsed CSS stylesheet
	:rtype: dict
	"""
	
	if stylesheet.errors:
		raise ValueError(stylesheet.errors[0])
	
	styles = {}
	
	# Remove declarations for other platforms and make
	
	for rule in stylesheet.rules:
		styles[rule.selector.as_css()] = {}
		
		for declaration in rule.declarations:
			name = declaration.name
			value = declaration.value.as_css()
			
			if "color" in name:
				value.lstrip("wx.")
				
				if value.startswith("SYS_COLOUR"):
					# wx.SystemColour
					if value in sys_colour_lookup:
						value = wx.SystemSettings.GetColour(sys_colour_lookup[value])
					else:
						raise ValueError(f"""wx.SystemColour 'value' not recognised.
See https://wxpython.org/Phoenix/docs/html/wx.SystemColour.enumeration.html for the list of valid values""")
				
				elif value.startswith("#"):
					# Hex value, pass to wx.Colour directly
					pass
				
				elif value.startswith("rgb("):
					# RGB value, convert to hex
					rgb_triplet = (int(x) for x in value.lstrip("rgb(").rstrip(")").split(","))
					value = webcolors.rgb_to_hex(rgb_triplet)
				
				else:
					# Named colour, convert to hex
					value = webcolors.name_to_hex(value)
			
			if name == "font-family":
				if value == "decorative":
					value = wx.FONTFAMILY_DECORATIVE
				
				elif value == "roman":
					value = wx.FONTFAMILY_ROMAN
				
				elif value == "script":
					value = wx.FONTFAMILY_SCRIPT
				
				elif value == "swiss":
					value = wx.FONTFAMILY_SWISS
				
				elif value == "modern":
					value = wx.FONTFAMILY_MODERN
				
				else:
					value = wx.FONTFAMILY_DEFAULT
			
			elif name == "font-style":
				if value == "slant":
					value = wx.FONTSTYLE_SLANT
				elif value == "italic":
					value = wx.FONTSTYLE_ITALIC
				else:
					value = wx.FONTSTYLE_NORMAL
			
			elif name == "font-weight":
				if value == "light":
					value = wx.FONTWEIGHT_LIGHT
				elif value == "bolt":
					value = wx.FONTWEIGHT_BOLD
				else:
					value = wx.FONTWEIGHT_NORMAL
			
			styles[rule.selector.as_css()][name] = value
	
	# if li p is not styled, use the default values
	if "li p" not in styles:
		styles["li p"] = text_defaults.copy()
	
	# If li p is missing any parameters, add them from the default values
	styles["li p"] = {**text_defaults, **styles["li p"]}
	
	# if li p::selection is not styled, use the values from p
	if "li p::selection" not in styles:
		styles["li p::selection"] = styles["li p"].copy()
	
	# If li p is missing any parameters, add them from p
	styles["li p::selection"] = {**styles["li p"], **styles["li p::selection"]}
	
	# For each li p class, add undefined values from p
	for style in styles:
		if style.split(".")[0] == "li p":
			if style == "li p":
				continue
			if "::selection" in style:
				continue
			styles[style] = {**styles["li p"], **styles[style]}
	
	# For each p::selection class, add undefined values from p::selection
	for style in styles:
		if style.split(".")[0] == "li p":
			if style == "li p::selection":
				continue
			if "::selection" not in style:
				continue
			styles[style] = {**styles["li p::selection"], **styles[style]}
	
	return styles





#!/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  logctrl.py
"""
Log Control, supporting text copying and zoom.
"""
#
#  Copyright (c) 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#  Based on PyCrust by Patrick K. O'Brien <pobrien@orbtech.com>
#   and https://wiki.wxpython.org/StyledTextCtrl%20Log%20Window%20Demo
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#


# stdlib
import keyword
import os

# 3rd party
import wx
from wx import stc

# this package
from domdf_wxpython_tools.utils import generate_faces
from domdf_wxpython_tools.keyboard import gen_keymap


# IDs
ID_WRAP = wx.NewIdRef()
ID_SHOW_LINENUMBERS = wx.NewIdRef()
ID_ZOOM_IN = wx.NewIdRef()
ID_ZOOM_OUT = wx.NewIdRef()
ID_ZOOM_DEFAULT = wx.NewIdRef()
ID_ZOOM_SET = wx.NewIdRef()


# Key bindings:
# Home              Go to the beginning of the line.
# Shift+Home        Select to the beginning of the command or line.
# Shift+End         Select to the end of the line.
# End               Go to the end of the line.
# Ctrl+C            Copy selected text
# Ctrl+]            Increase font size.
# Ctrl+[            Decrease font size.
# Ctrl+=            Default font size.
# Ctrl+F            Search
# TODO: F3                Search next
#

class LogCtrl(stc.StyledTextCtrl):
	"""
	Log Window based on StyledTextCtrl.
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.CLIP_CHILDREN | wx.SUNKEN_BORDER, name="Log"
			):
		"""
		:param parent: The parent window.
		:type parent: wx.Window
		:param id: An identifier for the Log window. wx.ID_ANY is taken to mean a default.
		:type id: wx.WindowID, optional
		:param pos: The Log window position. The value ::wxDefaultPosition indicates a default position, chosen by either the windowing system or wxWidgets, depending on platform.
		:type pos: wx.Point, optional
		:param size: The Log window size. The value ::wxDefaultSize indicates a default size, chosen by either the windowing system or wxWidgets, depending on platform.
		:type size: wx.Size, optional
		:param style: The window style. See wxPanel.
		:type style: int, optional
		:param name: Window name.
		:type name: str, optional
		"""
		
		stc.StyledTextCtrl.__init__(self, parent, id, pos, size, style, name)
		
		self._FACES = generate_faces()
		self._keyMap = gen_keymap()
		self._config()
		self.default_zoom = self.GetZoom()
		self._styles = [None] * 32
		self._free = 1
		
		# dispatcher.connect(receiver=self._fontsizer, signal='FontIncrease')
		# dispatcher.connect(receiver=self._fontsizer, signal='FontDecrease')
		# dispatcher.connect(receiver=self._fontsizer, signal='FontDefault')
		
		self.findDlg = None
		self.findData = wx.FindReplaceData()
		self.findData.SetFlags(wx.FR_DOWN)
		
		self.Bind(wx.EVT_CONTEXT_MENU, self.OnContextMenu)
		self.Bind(wx.EVT_FIND_CLOSE, self.OnFindClose)
		self.Bind(wx.EVT_KEY_DOWN, self.onKeyPress)
		self.Bind(wx.EVT_MENU, lambda event: self.Copy(), id=wx.ID_COPY)
		self.Bind(wx.EVT_MENU, lambda event: self.SelectAll(), id=wx.ID_SELECTALL)
		self.Bind(wx.EVT_MENU, self.OnFindText, id=wx.ID_FIND)
		#self.Bind(wx.EVT_MENU, self.OnWrap, id=ID_WRAP)
		self.Bind(wx.EVT_MENU, self.ToggleWrap, id=ID_WRAP)
		#self.Bind(wx.EVT_MENU, self.OnShowLineNumbers, id=ID_SHOW_LINENUMBERS)
		self.Bind(wx.EVT_MENU, self.ToggleLineNumbers, id=ID_SHOW_LINENUMBERS)
		self.Bind(wx.EVT_MENU, self.OnZoomIn, id=ID_ZOOM_IN)
		self.Bind(wx.EVT_MENU, self.OnZoomOut, id=ID_ZOOM_OUT)
		self.Bind(wx.EVT_MENU, self.OnZoomDefault, id=ID_ZOOM_DEFAULT)
		# TODO: Default Zoom and Set Zoom
		
		# Display the introductory banner information.
		self.AppendText('''Click "▶ Run Comparison" to start
		
Right click for options
''')
		# TODO: Make this not specific to GSMatch
		
		wx.CallAfter(self.ScrollToLine, 0)
	
	def _config(self):
		self.wrap()
		self.setDisplayLineNumbers(False)
		
		self.SetLexer(stc.STC_LEX_PYTHON)
		self.SetKeyWords(0, ' '.join(keyword.kwlist))
		
		self.setStyles(self._FACES)
		self.SetViewWhiteSpace(False)
		self.SetWrapMode(True)
		try:
			self.SetEndAtLastLine(False)
		except AttributeError:
			pass
		
		self.SetMargins(5, 5)
	
	def onKeyPress(self, event):
		"""Event Handler for key being pressed"""
		keycode = event.GetKeyCode()
		keyname = self._keyMap.get(keycode, None)
		modifiers = ""
		for mod, ch in (
				(event.ControlDown(), 'Ctrl+'),
				(event.AltDown(), 'Alt+'),
				(event.ShiftDown(), 'Shift+'),
				(event.MetaDown(), 'Meta+')
				):
			if mod:
				modifiers += ch
		
		if keyname is None:
			if 27 < keycode < 256:
				keyname = chr(keycode)
			else:
				keyname = "(%s)unknown" % keycode
		
		combination = modifiers + keyname
		
		commands = {
				"Ctrl+A": self.SelectAll,
				"Ctrl+C": self.Copy,
				"Ctrl+F": self.OnFindText,
				"UP": self.LineUp,
				"DOWN": self.LineDown,
				"RIGHT": self.CharRight,
				"LEFT": self.CharLeft,
				"Ctrl+RIGHT": self.WordRight,
				"Ctrl+LEFT": self.WordLeft,
				"END": self.LineEnd,
				"Shift+END": self.LineEndExtend,
				"HOME": self.Home,
				"Shift+HOME": self.HomeExtend,
				"PAGEDOWN": self.PageDown,
				"PAGEUP": self.PageUp,
				"Shift+PAGEDOWN": self.PageDownExtend,
				"Shift+PAGEUP": self.PageUpExtend,
				"Shift+LEFT": self.CharLeftExtend,
				"Shift+RIGHT": self.CharRightExtend,
				"Shift+UP": self.LineUpExtend,
				"Shift+DOWN": self.LineDownExtend,
				"Ctrl+Shift+LEFT": self.WordLeftExtend,
				"Ctrl+Shift+RIGHT": self.WordRightExtend,
				"Ctrl+]": self.ZoomIn,
				"Ctrl+[": self.ZoomOut,
				# "ESCAPE": here we should remove focus from the widget,
				"Ctrl+W": self.ToggleWrap,
				"Ctrl+L": self.ToggleLineNumbers,
				}
		
		if combination in commands:
			commands[combination]()
		elif combination == "Ctrl+=":
			self.SetZoom(self.default_zoom)
		
		print(combination)
	
	def fixLineEndings(self, text):
		"""
		Return text with line endings replaced by OS-specific endings.

		:param text:
		:type text:

		:return:
		:rtype:
		"""
		
		lines = text.split('\r\n')
		for l in range(len(lines)):
			chunks = lines[l].split('\r')
			for c in range(len(chunks)):
				chunks[c] = os.linesep.join(chunks[c].split('\n'))
			lines[l] = os.linesep.join(chunks)
		text = os.linesep.join(lines)
		return text
	
	def setStyles(self, faces):
		"""
		Configure font size, typeface and color for lexer.
		
		:param faces:
		:type faces:
		
		:return:
		:rtype:
		"""
		
		# Default style
		self.StyleSetSpec(
				stc.STC_STYLE_DEFAULT,
				"face:{mono},size:{size:d},back:{backcol}".format(**faces)
				)
		
		self.StyleClearAll()
		self.SetSelForeground(True, wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT))
		self.SetSelBackground(True, wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT))
		
		styles = [  # Built in styles
			(stc.STC_STYLE_LINENUMBER, f"back:#C0C0C0,face:{faces}(mono)s,size:{faces}(lnsize)d"),
			(stc.STC_STYLE_CONTROLCHAR, f"face:{faces}(mono)s"),
			(stc.STC_STYLE_BRACELIGHT, "fore:#0000FF,back:#FFFF88"),
			(stc.STC_STYLE_BRACEBAD, "fore:#FF0000,back:#FFFF88"),
			# Python styles
			(stc.STC_P_DEFAULT, f"face:{faces}(mono)s"),
			(stc.STC_P_COMMENTLINE, f"fore:#007F00,face:{faces}(mono)s"),
			(stc.STC_P_NUMBER, ""),
			(stc.STC_P_STRING, f"fore:#7F007F,face:{faces}(mono)s"),
			(stc.STC_P_CHARACTER, f"fore:#7F007F,face:{faces}(mono)s"),
			# (stc.STC_P_WORD, "fore:#00007F,bold"),
			(stc.STC_P_TRIPLE, "fore:#7F0000"),
			(stc.STC_P_TRIPLEDOUBLE, "fore:#000033,back:#FFFFE8"),
			# (stc.STC_P_CLASSNAME, "fore:#0000FF,bold"),
			# (stc.STC_P_DEFNAME, "fore:#007F7F,bold"),
			(stc.STC_P_OPERATOR, ""),
			(stc.STC_P_IDENTIFIER, ""),
			(stc.STC_P_COMMENTBLOCK, "fore:#7F7F7F"),
			(stc.STC_P_STRINGEOL, f"fore:#000000,face:{faces}(mono)s,back:#E0C0E0,eolfilled"),
			]
		
		for style in styles:
			self.StyleSetSpec(*style)
	
	def getStyle(self, c='black'):
		"""
		Returns a style for a given colour if one exists.  If no style
		exists for the colour, make a new style.

		If we run out of styles, (only 32 allowed here) we go to the top
		of the list and reuse previous styles.

		"""
		free = self._free
		if c and isinstance(c, str):
			c = c.lower()
		else:
			c = 'black'
		
		try:
			style = self._styles.index(c)
			return style
		
		except ValueError:
			style = free
			self._styles[style] = c
			self.StyleSetForeground(style, wx.NamedColour(c))
			
			free += 1
			if free > 31:
				free = 0
			self._free = free
			return style

	def OnZoomIn(self, *args):
		"""Event Handler for zooming in"""
		self.ZoomIn()
	
	def OnZoomOut(self, *args):
		"""Event Handler for zooming out"""
		self.ZoomOut()
	
	def OnZoomDefault(self, *args):
		"""Event Handler for resetting the zoom"""
		self.SetZoom(self.default_zoom)

	def setDisplayLineNumbers(self, state):
		self.lineNumbers = state
		if state:
			self.SetMarginType(1, stc.STC_MARGIN_NUMBER)
			self.SetMarginWidth(1, 40)
		else:
			# Leave a small margin so the feature hidden lines marker can be seen
			self.SetMarginType(1, 0)
			self.SetMarginWidth(1, 10)
	
	# def OnShowLineNumbers(self, event):
	# 	print("sln")
	# 	if hasattr(self, 'lineNumbers'):
	# 		self.lineNumbers = event.IsChecked()
	# 		self.setDisplayLineNumbers(self.lineNumbers)
	
	def ToggleLineNumbers(self, *args):
		self.setDisplayLineNumbers(not self.lineNumbers)

	def CanCopy(self):
		"""
		Returns True if text is selected and can be copied, False otherwise.
		
		:rtype: bool
		"""
		return self.GetSelectionStart() != self.GetSelectionEnd()

	def Copy(self):
		"""
		Copy the selection and place it on the clipboard.
		"""
		
		if self.CanCopy():
			command = self.GetSelectedText()
			data = wx.TextDataObject(command)
			self._clip(data)

	def _clip(self, data):
		"""
		Internal function for copying to clipboard
		"""
		
		if wx.TheClipboard.Open():
			wx.TheClipboard.UsePrimarySelection(False)
			wx.TheClipboard.SetData(data)
			wx.TheClipboard.Flush()
			wx.TheClipboard.Close()

	def wrap(self, wrap=True):
		"""
		Set whether text is word wrapped.
		
		:param wrap: Whether the text should be word wrapped
		:type wrap: bool
		"""
		
		try:
			self.SetWrapMode(wrap)
		except AttributeError:
			return 'Wrapping is not available in this version.'
	#
	# def OnWrap(self, event):
	# 	self.SetWrapMode(event.IsChecked())
	
	def ToggleWrap(self, *args):
		"""
		Toggle word wrap
		"""
		
		self.SetWrapMode(not self.GetWrapMode())
	
	def GetContextMenu(self):
		"""
		Create and return a context menu for the log.
		This is used instead of the scintilla default menu
		in order to correctly respect our immutable buffer.
		"""
		
		menu = wx.Menu()
		menu.Append(wx.ID_COPY, "&Copy")
		menu.Append(wx.ID_SELECTALL, "Select &All \tCtrl+A")
		menu.AppendSeparator()
		menu.Append(
				wx.ID_FIND, '&Find \tCtrl+F',
				'Search for text in the log'
				)
		menu.Append(
				ID_WRAP, '&Wrap Lines\tCtrl+W',
				'Wrap lines at right edge', wx.ITEM_CHECK
				)
		menu.Append(
				ID_SHOW_LINENUMBERS, '&Show Line Numbers\tCtrl+L',
				'Show Line Numbers', wx.ITEM_CHECK
				)
		menu.AppendSeparator()
		menu.Append(ID_ZOOM_IN, 'Zoom &In\tCtrl+]', 'Zoom In')
		menu.Append(ID_ZOOM_OUT, 'Zoom &Out\tCtrl+[', 'Zoom Out')
		menu.Append(ID_ZOOM_DEFAULT, '&Reset Zoom\tCtrl+=', 'Zoom Out')
		menu.Append(ID_ZOOM_SET, 'Set &Zoom', 'Zoom Out')
		
		menu.Check(ID_WRAP, self.GetWrapMode())
		menu.Check(ID_SHOW_LINENUMBERS, self.lineNumbers)
		
		return menu
	
	def OnContextMenu(self, event):
		"""Event Handler for showing the context menu"""
		menu = self.GetContextMenu()
		self.PopupMenu(menu)


	# Find Dialog Methods
	def GetLastPosition(self):
		return self.GetLength()
	
	def GetRange(self, start, end):
		return self.GetTextRange(start, end)
	
	def GetSelection(self):
		return self.GetAnchor(), self.GetCurrentPos()
	
	def ShowPosition(self, pos):
		line = self.LineFromPosition(pos)
		# self.EnsureVisible(line)
		self.GotoLine(line)
	
	def DoFindNext(self, findData, findDlg=None):
		backward = not (findData.GetFlags() & wx.FR_DOWN)
		matchcase = (findData.GetFlags() & wx.FR_MATCHCASE) != 0
		end = self.GetLength()
		# Changed to reflect the fact that StyledTextControl is in UTF-8 encoding
		textstring = self.GetTextRange(0, end).encode('utf-8')
		findstring = findData.GetFindString().encode('utf-8')
		if not matchcase:
			textstring = textstring.lower()
			findstring = findstring.lower()
		if backward:
			start = self.GetSelection()[0]
			loc = textstring.rfind(findstring, 0, start)
		else:
			start = self.GetSelection()[1]
			loc = textstring.find(findstring, start)
		
		# if it wasn't found then restart at begining
		if loc == -1 and start != 0:
			if backward:
				start = end
				loc = textstring.rfind(findstring, 0, start)
			else:
				start = 0
				loc = textstring.find(findstring, start)
		
		# was it still not found?
		if loc == -1:
			dlg = wx.MessageDialog(self, 'Unable to find the search text.',
								   'Not found!',
								   wx.OK | wx.ICON_INFORMATION)
			dlg.ShowModal()
			dlg.Destroy()
		if findDlg:
			if loc == -1:
				wx.CallAfter(findDlg.SetFocus)
				return
			else:
				findDlg.Close()
		
		# show and select the found text
		self.ShowPosition(loc)
		self.SetSelection(loc, loc + len(findstring))
	
	def OnFindText(self, *args):
		if self.findDlg is not None:
			return
		
		self.findDlg = wx.FindReplaceDialog(self, self.findData,
											"Find", wx.FR_NOWHOLEWORD)
		self.findDlg.Show()
	
	def OnFindClose(self, event):
		self.findDlg.Destroy()
		self.findDlg = None


	# Save Methods
	def bufferHasChanged(self):
		# the log buffers can always be saved
		return True
	
	def bufferSave(self):
		import time
		appname = wx.GetApp().GetAppName()
		default = appname + '-' + time.strftime("%Y%m%d-%H%M.py")
		filename = wx.FileSelector(
				"Save File As", "Saving",
				default_filename=default,
				default_extension="py",
				wildcard="*.py",
				flags=wx.SAVE | wx.OVERWRITE_PROMPT
				)
		
		if not filename:
			return
		
		text = self.GetText()
		
		try:
			f = open(filename, "w")
			f.write(text)
			f.close()
		except:  # TODO: Find error type
			d = wx.MessageDialog(
					self, u'Error saving session', u'Error',
					wx.OK | wx.ICON_ERROR
					)
			
			d.ShowModal()
			d.Destroy()
		
	def write(self, text):
		"""
		Display text in the log.

		Replace line endings with OS-specific endings.
		
		:param text:
		:type text:
		"""
		
		text = self.fixLineEndings(text)
		self.AddText(text)
		self.EnsureCaretVisible()

	def Append(self, text, c=None):
		"""
		Add the text to the end of the control using colour c which
		should be suitable for feeding directly to wx.NamedColour.

		:param text: ... Should be a unicode string or contain only ascii data.
		:type text:
		:param c:
		:type c:
		"""
		
		style = self.getStyle(c)
		lenText = len(text.encode('utf8'))
		end = self.GetLength()
		self.AppendText(text)
		self.StartStyling(end, 31)
		self.SetStyling(lenText, style)
		self.EnsureCaretVisible()
	
	def AppendStderr(self, text):
		"""
		Add the stderr text to the end of the control using colour "red"

		:param text: ... Should be a unicode string or contain only ascii data.
		:type text:
		"""
		
		self.Append(text, "red")






#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  timer_thread.py
"""
Background thread that sends an event after the specified interval.
Useful for timeouts or updating timers, clocks etc.


Includes code from https://gist.github.com/samarthbhargav/5a515a399f7113137331
"""
#
#  Copyright (c) 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import time

from threading import Thread, Event

from domdf_wxpython_tools.events import SimpleEvent

timer_event = SimpleEvent("Timer")


class Timer(Thread):
	"""
	Background Timer Class
	"""
	
	def __init__(self, parent, interval=1.0):
		"""
		Init Timer Thread Class

		:param parent: Class to send event updates to
		:param interval: Interval to trigger events at, in seconds. Default 1.0
		:type interval: float
		"""
		
		self._stopevent = Event()
		Thread.__init__(self, name="TimerThread")
		self._parent = parent
		self._interval = interval
	
	def run(self):
		"""
		Run Timer Thread
		"""
		
		wait_time = 0 + self._interval
		while not self._stopevent.isSet():
			time.sleep(0.1)
			wait_time -= 0.1
			if wait_time < 0.0:
				timer_event.trigger()
				wait_time = 0 + self._interval
	
	def join(self, timeout=None):
		"""
		Stop the thread and wait for it to end

		:param timeout:
		:type:
		"""
		
		self._stopevent.set()
		Thread.join(self, timeout)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  list_dialog.py
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
# generated by wxGlade 0.9.3 on Wed Jun 12 18:14:39 2019
#

import wx

# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


class list_dialog(wx.Dialog):
	"""
	A dialog containing a wx.ListBox
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY,
			title='Choose', label="Choose: ", choices=None,
			pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.DEFAULT_DIALOG_STYLE, name=wx.DialogNameStr
			):
		
		"""
		:param parent: The parent window. Can be None, a frame or another dialog box
		:type parent: wx.Window
		:param id: An identifier for the dialog. wx.ID_ANY is taken to mean a default.
		:type id: wx.WindowID, optional
		:param title: The title of the dialog
		:type title: str, optional
		:param label: The label for the wx.ListBox
		:type label: str, optional
		:param choices: A list of choices for the wx.ListBox
		:type choices: list of str
		:param pos: The dialog position. The value ::wxDefaultPosition indicates a default position, chosen by either the windowing system or wxWidgets, depending on platform.
		:type pos: wx.Point, optional
		:param size: The dialog size. The value ::wxDefaultSize indicates a default size, chosen by either the windowing system or wxWidgets, depending on platform.
		:type size: wx.Size, optional
		:param style: The window style. See wxPanel.
		:type style: int, optional
		:param name: Window name.
		:type name: str, optional
		"""
		
		if choices is None:
			choices = [
					"egg and bacon",
					"egg sausage and bacon",
					"egg and spam",
					"egg bacon and spam",
					"egg bacon sausage and spam",
					"spam bacon sausage and spam",
					"spam egg spam spam bacon and spam",
					"spam sausage spam spam bacon spam tomato and spam",
					"spam spam spam egg and spam",
					"spam spam spam spam spam spam baked beans spam spam spam",
					]
			
		self.title = title
		self.label = label
		self.choices = choices
		
		args = (parent, id)
		kwds = dict(pos=pos, size=size, style=style, name=name)
		
		# begin wxGlade: list_dialog.__init__
		kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_DIALOG_STYLE
		wx.Dialog.__init__(self, *args, **kwds)
		self.SetSize((512, -1))
		self.main_panel = wx.Panel(self, wx.ID_ANY)
		self.list_box = wx.ListBox(self.main_panel, wx.ID_ANY, choices=[])
		self.cancel_btn = wx.Button(self.main_panel, wx.ID_CANCEL, "")
		self.select_btn = wx.Button(self.main_panel, wx.ID_ANY, "Select")
		
		self.__set_properties()
		self.__do_layout()
		
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.do_select, self.list_box)
		self.Bind(wx.EVT_BUTTON, self.do_select, self.select_btn)
		# end wxGlade
		self.list_box.Clear()
		self.list_box.AppendItems(choices)
	
	def __set_properties(self):
		# begin wxGlade: list_dialog.__set_properties
		self.SetTitle("Choose")
		self.SetSize((512, -1))
		# end wxGlade
		self.SetTitle(self.title)
	
	def __do_layout(self):
		# begin wxGlade: list_dialog.__do_layout
		parent_sizer = wx.BoxSizer(wx.VERTICAL)
		main_sizer = wx.BoxSizer(wx.VERTICAL)
		btn_sizer = wx.GridSizer(1, 2, 0, 7)
		borders_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Choose: ")
		borders_label.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, 0, ""))
		main_sizer.Add(borders_label, 0, wx.BOTTOM, 7)
		main_sizer.Add(self.list_box, 1, wx.EXPAND, 0)
		static_line_10 = wx.StaticLine(self.main_panel, wx.ID_ANY)
		main_sizer.Add(static_line_10, 0, wx.BOTTOM | wx.EXPAND | wx.TOP, 5)
		btn_sizer.Add(self.cancel_btn, 0, wx.ALIGN_CENTER, 0)
		btn_sizer.Add(self.select_btn, 0, wx.ALIGN_CENTER, 0)
		main_sizer.Add(btn_sizer, 0, wx.ALIGN_BOTTOM | wx.ALIGN_RIGHT, 0)
		self.main_panel.SetSizer(main_sizer)
		parent_sizer.Add(self.main_panel, 1, wx.ALL | wx.EXPAND, 10)
		self.SetSizer(parent_sizer)
		self.Layout()
		# end wxGlade
		borders_label.SetLabel(self.label)
	
	def do_select(self, event):  # wxGlade: list_dialog.<event_handler>
		"""Event Handler for item in list being selected"""
		self.EndModal(wx.ID_OK)
	
	def do_cancel(self, event):  # wxGlade: list_dialog.<event_handler>
		"""Event Handler for dialog being cancelled"""
		self.Destroy()

# end of class list_dialog



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  utils.py
#
#  Copyright 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import wx


def toggle(object):
	if object.IsEnabled():
		object.SetValue(not object.GetValue())
		return True
	return False


def coming_soon(msg="This feature has not been implemented yet"):
	wx.MessageBox(msg, "Coming Soon", wx.ICON_INFORMATION | wx.OK)


def collapse_label(text, collapsed=True):
	import sys
	if sys.platform == "win32":
		return f"{text} {'>>' if collapsed else '<<'}"
	else:
		return f"{'⯈' if collapsed else '⯆'} {text}"


def generate_faces():
	FACES = {
			'backcol': '#FFFFFF',
			'calltipbg': '#FFFFB8',
			'calltipfg': '#404040',
			'size': 12,
			'lnsize': 10,
			'other': 'new century schoolbook',
			'times': 'Times',
			'mono': 'Courier',
			'helv': 'Helvetica',
			}
	
	if 'wxMSW' in wx.PlatformInfo:
		FACES = {
				**FACES,
				'times': 'Times New Roman',
				'mono': 'Courier New',
				'helv': 'Arial',
				'lucida': 'Lucida Console',
				'other': 'Comic Sans MS',
				'size': 10,
				'lnsize': 8,
				}
	
	elif 'wxGTK' in wx.PlatformInfo and (
			'gtk2' in wx.PlatformInfo or 'gtk3' in wx.PlatformInfo
			):
		FACES = {
				**FACES,
				'times': 'Serif',
				'mono': 'Monospace',
				'helv': 'Sans',
				'size': 10,
				'lnsize': 9,
				 }
	
	elif 'wxMac' in wx.PlatformInfo:
		FACES = {
				**FACES,
				'times': 'Lucida Grande',
				'mono': 'Monaco',
				'helv': 'Geneva',
				}
	
	return FACES





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  dialogs.py
"""
Several dialog classes and helper functions for file/folder dialogs
"""
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# stdlib
import os

# 3rd party
import wx

# this package
from domdf_wxpython_tools.validators import CharValidator
from domdf_wxpython_tools.list_dialog import list_dialog

# style enumeration
style_uppercase = 4
style_lowercase = 8
style_hidden = 16


common_filetypes = {
		"jpeg": ("JPEG files", ["jpg", "jpeg"]),
		"png": ("PNG files", ["png"]),
		"bmp": ("BMP files", ["bmp"]),
		"tiff": ("TIFF files", ["tiff", "tif"]),
		"gif": ("GIF files", ["gif"])
		# TODO: Add more
		}


def file_dialog_wildcard(parent, title, wildcard, style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT, **kwargs):
	"""
	Create a wx.FileDialog with the wildcard string given, and return a list of the files selected.
	
	:param parent:
	:type parent:
	:param wildcard:
	:type wildcard:
	:param title:
	:type title:
	:param style:
	:type style:
	:param kwargs:
	:type kwargs:

	:return: List of filenames for the selected files. If wx.FD_MULTIPLE is not in the style, the list will contain only one element
	:rtype: list of str
	"""
	
	with wx.FileDialog(
			parent, title,
			wildcard=wildcard,
			style=style, **kwargs
			) as fileDialog:
		
		if fileDialog.ShowModal() == wx.ID_CANCEL:
			return  # the user changed their mind
		
		try:
			pathnames = fileDialog.GetPaths()
		except:
			pathnames = [fileDialog.GetPath()]
		
		filter_extension_list = wildcard.split("|")[1::2]
		valid_extensions = [os.path.splitext(ext)[1] for ext in ";".join(filter_extension_list).split(";")]
		selected_filter_index = fileDialog.GetFilterIndex()
		
		selected_filter_extensions = filter_extension_list[selected_filter_index].replace("*.", ".").split(";")
		
		for index, pathname in enumerate(pathnames):
			if selected_filter_extensions[0] != ".*":
				if os.path.splitext(pathname)[-1].lower() not in selected_filter_extensions:
					pathnames[index] = pathname + f"{selected_filter_extensions[0]}"
		
		return pathnames


def file_dialog_multiple(parent, extension, title, filetypestring, style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT, **kwargs):
	"""
	Create a wx.FileDialog with for the extension and filetypestring given, and return a list of the files selected.
	
	:param parent:
	:type parent:
	:param extension:
	:type extension:
	:param title:
	:type title:
	:param filetypestring:
	:type filetypestring:
	:param style:
	:type style:
	:param kwargs:
	:type kwargs:
	
	:return: List of filenames for the selected files. If wx.FD_MULTIPLE is not in the style, the list will contain only one element
	:rtype: list of str
	"""
	
	with wx.FileDialog(
			parent, title,
			wildcard=f"{filetypestring} (*.{extension.lower()})|*.{extension.lower()};*.{extension.upper()}",
			style=style, **kwargs
	) as fileDialog:
		
		if fileDialog.ShowModal() == wx.ID_CANCEL:
			return  # the user changed their mind
		
		try:
			pathnames = fileDialog.GetPaths()
		except:
			pathnames = [fileDialog.GetPath()]
		
		for index, pathname in enumerate(pathnames):
			if extension != "*":
				if os.path.splitext(pathname)[-1].lower() != f".{extension}":
					pathnames[index] = pathname + f".{extension}"
		# else:
		# 	pathnames[index] = os.path.splitext(pathname)[0]
		
		return pathnames


def file_dialog(*args, **kwargs):
	"""
	Create a wx.FileDialog with for the extension and filetypestring given, and return the filename selected.
	
	:param parent:
	:type parent:
	:param extension:
	:type extension:
	:param title:
	:type title:
	:param filetypestring:
	:type filetypestring:
	:param style:
	:type style:
	:param kwargs:
	:type kwargs:
	
	:return: The filename selected in the dialog. If wx.FD_MULTIPLE is in the style, this will be the first filename selected.
	:rtype: str
	"""
	
	paths = file_dialog_multiple(*args, **kwargs)
	
	if paths is not None:
		return paths[0]


class FloatEntryDialog(wx.TextEntryDialog):
	"""
	Alternative to wx.NumberEntryDialog that provides a TextCtrl which only allows numbers and decimal points to be entered.
	
	Based on http://wxpython-users.1045709.n5.nabble.com/Adding-Validation-to-wx-TextEntryDialog-td2371082.html
	"""
	
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.textctrl = self.FindWindowById(3000)
		
		self.textctrl.SetValidator(CharValidator("float-only"))
	
	def GetValue(self):
		value = self.textctrl.GetValue()
		if value == '':
			return None
		else:
			return float(value)


class IntEntryDialog(wx.TextEntryDialog):
	"""
	Alternative to wx.NumberEntryDialog that provides a TextCtrl which only allows numbers to be entered.

	Based on http://wxpython-users.1045709.n5.nabble.com/Adding-Validation-to-wx-TextEntryDialog-td2371082.html
	"""
	
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.textctrl = self.FindWindowById(3000)
		
		self.textctrl.SetValidator(CharValidator("int-only"))

	def GetValue(self):
		value = self.textctrl.GetValue()
		if value == '':
			return None
		else:
			return int(value)


class Wildcards:
	"""
	Class to generate glob wildcards for wx.FileDialog
	"""
	
	def __init__(self):
		self._wildcards = []
	
	def add_filetype(
			self, description, extensions=None, hint_format=style_lowercase,
			value_format=style_lowercase | style_uppercase):
		"""
		Add a filetype to the wildcards
		
		:param description: Description of the filetype
		:type description: str
		:param extensions: A list of valid file extensions for the filetype
		:type extensions: list of str
		:param hint_format: How the hints should be formatted.
		:type hint_format: int
		:param value_format: How the values should be formatted.
		:type value_format:	int
		
		Valid values for `hint_format` and `value_format` are `style_uppercase`,
		`style_lowercase` and `style_hidden`, which can be combined using the `|` operator.
		"""
		
		if extensions:
			hint = []
			value = []
			
			for extension in extensions:
				
				if not extension.startswith("*."):
					extension = f"*.{extension}"
				
				# Hint
				if not hint_format & style_hidden:
					if hint_format & style_lowercase:
						hint.append(extension.lower())
						
					if hint_format & style_uppercase:
						hint.append(extension.upper())
					
				# Value
				if value_format & style_lowercase:
					value.append(extension.lower())
						
				if value_format & style_uppercase:
					value.append(extension.upper())
			
			if hint_format & style_hidden:
				self._wildcards.append(f"{description}|{';'.join(value)}")
			else:
				self._wildcards.append(f"{description} ({';'.join(hint)})|{';'.join(value)}")
		
		else:
			self._wildcards.append(f"{description}")
	
	@property
	def wildcard(self):
		"""
		Returns a string representing the wildcards for use in wx.FileDialog or file_dialog_wildcards
		
		:rtype: str
		"""
		
		return "|".join(self._wildcards)
	
	def add_common_filetype(
			self, filetype, hint_format=style_lowercase,
			value_format=style_lowercase | style_uppercase):
		"""
		Add a common filetype.
		
		:param filetype: The name of the filetype, Possible values are in common_filetypes
		:type filetype: str
		:param hint_format: How the hints should be formatted.
		:type hint_format: int
		:param value_format: How the values should be formatted.
		:type value_format:	int
		
		Valid values for `hint_format` and `value_format` are `style_uppercase`,
		`style_lowercase` and `style_hidden`, which can be combined using the `|` operator.
		"""
		
		self.add_filetype(*common_filetypes[filetype], hint_format=hint_format, value_format=value_format)

	def add_image_wildcard(self, value_format=style_lowercase | style_uppercase):
		"""
		Add a wildcard for all image filetypes.

		:param value_format: How the values should be formatted.
		:type value_format:	int

		Valid values for `value_format` are `style_uppercase`,
		`style_lowercase` and `style_hidden`, which can be combined using the `|` operator.
		"""
		
		image_extensions = []
		for key, item in common_filetypes.items():
			if key in {"jpeg", "png", "bmp", "tiff", "gif"}:
				for extension in item[1]:
					image_extensions.append(f"*.{extension.lower()}")
					image_extensions.append(f"*.{extension.upper()}")
				
		self.add_filetype("Image files", image_extensions, hint_format=style_hidden, value_format=value_format)
	
	def add_all_files_wildcard(self, hint_format=0):
		"""
		Add a wildcard for 'All Files'.

		:param hint_format: How the hints should be formatted. Valid values are None and `style_hidden`.
		:type hint_format: int
		"""
		
		if hint_format & style_hidden:
			self._wildcards.append("All files|*.*")
		else:
			self._wildcards.append("All files (*.*)|*.*")
	
	def __repr__(self):
		return f"Wildcard = {self.wildcard}"
	
	def __str__(self):
		return self.wildcard


# legacy name
FileDialogWildcards = Wildcards


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  __init__.py
"""
A custom Panel that acts as a ListCtrl for other wx.Panel objects.

An example ListItem exists that provides two StaticText fields and
can be used as the basis for custom list items
"""
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
# generated by wxGlade 0.9.2 on Thu Jan 16 16:34:51 2020
#

# stdlib
from pathlib import Path

# this package
from domdf_wxpython_tools.panel_listctrl.panel_listctrl import PanelListCtrl, PanelListItem
from domdf_wxpython_tools.panel_listctrl.css_parser import parse_css, parse_css_file
from domdf_wxpython_tools.panel_listctrl.font_parser import parse_font

default_css = Path(__file__).parent / "Default.css"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  ColourPickerPanel.py
"""
Based on StylePickerPanel, a Panel for selecting a list of colours, and their order
"""
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
# generated by wxGlade 0.9.3 on Tue Apr  9 18:07:58 2019
#


# 3rd party
import wx
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.figure import Figure
from domdf_wxpython_tools.StylePickerPanel import StylePickerPanel

# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


default_colours = [
		'#1f77b4',
		'#ff7f0e',
		'#2ca02c',
		'#d62728',
		'#9467bd',
		'#8c564b',
		'#e377c2',
		'#7f7f7f',
		'#bcbd22',
		'#17becf'
		]

default_picker_choices = [
		'#000000',
		'#ff0000',
		'#ffa500',
		'#00ff00',
		'#0000ff',
		'#551a8b',
		'#008080'
		]


class ColourPickerPanel(StylePickerPanel):
	"""
	Based on StylePickerPanel, a Panel for selecting a list of colours, and their order
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL, name=wx.PanelNameStr, label="Choose Colours: ",
			picker_choices=None, selection_choices=None,
			):
		"""
		:param parent: The parent window.
		:type parent: wx.Window
		:param id: An identifier for the panel. wx.ID_ANY is taken to mean a default.
		:type id: wx.WindowID, optional
		:param pos: The panel position. The value ::wxDefaultPosition indicates a default position, chosen by either the windowing system or wxWidgets, depending on platform.
		:type pos: wx.Point, optional
		:param size: The panel size. The value ::wxDefaultSize indicates a default size, chosen by either the windowing system or wxWidgets, depending on platform.
		:type size: wx.Size, optional
		:param style: The window style. See wxPanel.
		:type style: int, optional
		:param name: Window name.
		:type name: str, optional
		:param label: Label for the panel
		:type label: str, optional
		:param picker_choices: A list of hex value choices to populate the 'picker' size of the panel with
		:type picker_choices: list of str
		:param selection_choices: A list of hex value choices to populate the 'selection' size of the panel with
		:type selection_choices: list of str
		"""
		
		args = (parent, id, pos, size)
		kwds = {"style": style, "name": name}
		
		if picker_choices is None:
			picker_choices = default_picker_choices

		if selection_choices is None:
			selection_choices = default_colours[:]

		self.label = label
		self.picker_choices = picker_choices
		self.selection_choices = selection_choices
		
		# begin wxGlade: ColourPickerPanel.__init__
		kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
		wx.Panel.__init__(self, *args, **kwds)
		self.main_panel = wx.Panel(self, wx.ID_ANY)
		self.move_panel = wx.Panel(self.main_panel, wx.ID_ANY)
		self.picker_list_box = wx.ListBox(self.main_panel, wx.ID_ANY, choices=[])
		
		self.picker_figure = Figure()
		
		self.picker_canvas = FigureCanvas(self.main_panel, wx.ID_ANY, self.picker_figure)
		self.add_btn = wx.Button(self.main_panel, wx.ID_ANY, u"Add 🡲")
		self.remove_btn = wx.Button(self.main_panel, wx.ID_ANY, u"🡰 Remove")
		self.selection_list_box = wx.ListBox(self.main_panel, wx.ID_ANY, choices=[])
		self.up_btn = wx.Button(self.main_panel, wx.ID_ANY, u"🡱 Up")
		self.down_btn = wx.Button(self.main_panel, wx.ID_ANY, u"🡳 Down")
		
		self.selection_figure = Figure()
		
		self.selection_canvas = FigureCanvas(self.main_panel, wx.ID_ANY, self.selection_figure)

		self.__set_properties()
		self.__do_layout()
		# end wxGlade
		self.Bind(wx.EVT_LISTBOX, self.update_picker_preview, self.picker_list_box)
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.add, self.picker_list_box)
		self.Bind(wx.EVT_BUTTON, self.add, self.add_btn)
		self.Bind(wx.EVT_BUTTON, self.remove, self.remove_btn)
		self.Bind(wx.EVT_LISTBOX, self.update_selection_preview, self.selection_list_box)
		self.Bind(wx.EVT_BUTTON, self.move_up, self.up_btn)
		self.Bind(wx.EVT_BUTTON, self.move_down, self.down_btn)
		
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.remove, self.selection_list_box)
		
		self.remove_btn.SetLabel("Remove")
		
		self.selection_list_box.Clear()
		self.selection_list_box.AppendItems(self.selection_choices)
		if not self.selection_list_box.IsEmpty():
			self.selection_list_box.SetSelection(0)
		
		self.picker_list_box.Clear()
		self.picker_list_box.AppendItems(self.picker_choices)
		if not self.picker_list_box.IsEmpty():
			self.picker_list_box.SetSelection(0)
		
		self.picker_axes = self.picker_figure.add_subplot(111)
		self.selection_axes = self.selection_figure.add_subplot(111)
		self.update_picker_preview()
		self.update_selection_preview()

	def __set_properties(self):
		# begin wxGlade: ColourPickerPanel.__set_properties
		self.move_panel.SetMinSize((170, -1))
		self.picker_list_box.SetMinSize((170, 256))
		self.picker_canvas.SetMinSize((64, 64))
		self.selection_list_box.SetMinSize((170, 256))
		self.up_btn.SetMinSize((80, -1))
		self.down_btn.SetMinSize((80, -1))
		self.selection_canvas.SetMinSize((64, 64))
		self.main_panel.SetMinSize((450, -1))
		# end wxGlade

	def __do_layout(self):
		# begin wxGlade: ColourPickerPanel.__do_layout
		parent_sizer = wx.BoxSizer(wx.HORIZONTAL)
		main_sizer = wx.BoxSizer(wx.VERTICAL)
		list_grid_sizer = wx.BoxSizer(wx.HORIZONTAL)
		sizer_4 = wx.BoxSizer(wx.VERTICAL)
		selection_preview_sizer = wx.BoxSizer(wx.HORIZONTAL)
		move_grid = wx.GridSizer(1, 2, 0, 5)
		sizer_3 = wx.BoxSizer(wx.VERTICAL)
		sizer_2 = wx.BoxSizer(wx.VERTICAL)
		picker_preview_sizer = wx.BoxSizer(wx.HORIZONTAL)
		grid_sizer = wx.GridSizer(1, 3, 10, 10)
		borders_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Choose Styles: ")
		borders_label.SetMinSize((128, 20))
		borders_label.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, 0, ""))
		grid_sizer.Add(borders_label, 0, wx.BOTTOM, 7)
		grid_sizer.Add(self.move_panel, 1, wx.ALIGN_CENTER | wx.EXPAND, 0)
		main_sizer.Add(grid_sizer, 0, 0, 0)
		sizer_2.Add(self.picker_list_box, 5, wx.EXPAND, 0)
		sizer_2.Add((0, 0), 0, 0, 0)
		preview_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Preview: ")
		picker_preview_sizer.Add(preview_label, 0, 0, 0)
		picker_preview_sizer.Add(self.picker_canvas, 1, wx.EXPAND, 0)
		sizer_2.Add(picker_preview_sizer, 0, wx.EXPAND | wx.TOP, 5)
		list_grid_sizer.Add(sizer_2, 1, wx.EXPAND, 0)
		sizer_3.Add(self.add_btn, 0, wx.ALIGN_CENTER | wx.BOTTOM, 10)
		sizer_3.Add(self.remove_btn, 0, wx.ALIGN_CENTER, 0)
		list_grid_sizer.Add(sizer_3, 5, wx.ALIGN_CENTER | wx.BOTTOM, 64)
		sizer_4.Add(self.selection_list_box, 5, wx.EXPAND, 0)
		move_grid.Add(self.up_btn, 0, wx.ALIGN_CENTER, 0)
		move_grid.Add(self.down_btn, 0, wx.ALIGN_CENTER, 0)
		sizer_4.Add(move_grid, 0, wx.EXPAND, 0)
		preview_label_1 = wx.StaticText(self.main_panel, wx.ID_ANY, "Preview: ")
		selection_preview_sizer.Add(preview_label_1, 0, 0, 0)
		selection_preview_sizer.Add(self.selection_canvas, 1, wx.EXPAND, 0)
		sizer_4.Add(selection_preview_sizer, 0, wx.EXPAND | wx.TOP, 5)
		list_grid_sizer.Add(sizer_4, 1, wx.EXPAND, 0)
		main_sizer.Add(list_grid_sizer, 0, 0, 0)
		static_line_11 = wx.StaticLine(self.main_panel, wx.ID_ANY)
		main_sizer.Add(static_line_11, 0, wx.BOTTOM | wx.EXPAND | wx.TOP, 5)
		self.main_panel.SetSizer(main_sizer)
		parent_sizer.Add(self.main_panel, 0, wx.ALL, 10)
		self.SetSizer(parent_sizer)
		parent_sizer.Fit(self)
		self.Layout()
		# end wxGlade
		borders_label.SetLabel(self.label)
	
	def add(self, event):  # wxGlade: ColourPickerPanel.<event_handler>
		"""
		Event handler for adding the colour currently selected in the 'picker' to the 'selection'
		"""
		
		selection = self.picker_list_box.GetSelection()
		if selection == -1:
			return
		
		selection_string = self.picker_list_box.GetString(selection)
		if selection_string == '':
			return
		
		self.selection_list_box.Append(selection_string)
		
		self.update_picker_preview()
		
		self.selection_list_box.SetSelection(self.selection_list_box.GetCount() - 1)
		self.update_selection_preview()
		event.Skip()

	def remove(self, event):  # wxGlade: ColourPickerPanel.<event_handler>
		"""
		Event handler for removing the colour currently selected in the 'selection'
		"""
		
		selection = self.selection_list_box.GetSelection()
		if selection == -1:
			return
		
		selection_string = self.selection_list_box.GetString(selection)
		if selection_string == '':
			return
		
		self.selection_list_box.Delete(self.selection_list_box.GetSelection())
		
		self.update_selection_preview()
		event.Skip()

	def update_preview(self, list_obj, axes):
		"""
		Update the preview from the given list
		
		:param list_obj: The list to update the preview for
		:type list_obj: wx.ListBox
		:param axes: The preview axes to update
		:type axes: matplotlib.axes.Axes
		"""
		
		axes.clear()
		axes.axis('off')
		selection_string = list_obj.GetStringSelection()
		if selection_string == '':
			return
		
		axes.scatter(1, 1, s=400, color=selection_string, marker="s")
	
	def pick(self, *args):
		"""
		Open a wx.ColourDialog to edit the colour currently selected in the picker
		"""
		
		selection = self.selection_list_box.GetSelection()
		if selection == -1:
			return

		selection_string = self.selection_list_box.GetString(selection)
		if selection_string == '':
			return

		colour = wx.ColourData()
		colour.SetColour(selection_string)

		dlg = wx.ColourDialog(self, data=colour)

		res = dlg.ShowModal()
		if res == wx.ID_OK:
			self.selection_list_box.Delete(selection)
			self.selection_list_box.InsertItems(
					[dlg.GetColourData().GetColour().GetAsString(wx.C2S_HTML_SYNTAX)],
					selection
					)
			self.selection_list_box.SetSelection(selection)
			self.update_selection_preview()
			dlg.Destroy()
	
	def GetSelection(self):
		"""
		Returns a list of the currently selected colours
		
		:rtype: list of str
		"""
		return [
				self.selection_list_box.GetString(item) for item in
				range(self.selection_list_box.GetCount())
				]
	
	get_selection = GetSelection
	
# end of class ColourPickerPanel



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  picker.py
#
#  Copyright 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
# generated by wxGlade 0.9.3 on Thu Jun  6 12:07:28 2019
#

# stdlib
import pathlib

# 3rd party
import wx

# this package
from . import icons
from domdf_wxpython_tools.dialogs import file_dialog


# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade

# TODO: Forward events

class dir_picker(wx.Panel):
	"""
	
	"""
	def __init__(
			self, parent, id=wx.ID_ANY, value="", pos=wx.DefaultPosition,
			size=wx.DefaultSize, style=wx.TAB_TRAVERSAL, name=b"dir_picker",
	):
		"""
		
		:param parent:
		:type parent:
		:param id:
		:type id:
		:param value:
		:type value:
		:param pos:
		:type pos:
		:param size:
		:type size:
		:param style:
		:type style:
		:param name:
		:type name:
		"""
		
		self._parent = parent
		self.initial_value = value
		
		wx.Panel.__init__(self, self._parent, id=id, pos=pos, size=size, style=style, name=name)
		self.dir_value = wx.TextCtrl(self, wx.ID_ANY, "")
		self.clear_btn = wx.BitmapButton(self, wx.ID_ANY, icons.get_button_icon("ART_GO_BACK", 16))
		self.browse_btn = wx.BitmapButton(self, wx.ID_ANY, icons.get_button_icon("ART_FOLDER_OPEN", 16))
		
		self.textctrl_width = 150
		self.height = 29
		
		self.__set_properties()
		self.__do_layout()
		
		self.Bind(wx.EVT_BUTTON, self.Clear, self.clear_btn)
		self.Bind(wx.EVT_BUTTON, self.Browse, self.browse_btn)
		
		return
	
	def __set_properties(self):
		# begin wxGlade: dir_picker.__set_properties
		self.dir_value.SetMinSize((-1, 29))
		self.dir_value.SetMaxSize((-1, 29))
		self.clear_btn.SetMinSize((29, 29))
		self.browse_btn.SetMinSize((29, 29))
		# end wxGlade
		# self.dir_value.SetMinSize((self.textctrl_width, self.height))
		# self.clear_btn.SetMinSize((self.height, self.height))
		# self.browse_btn.SetMinSize((self.height, self.height))
		# self.SetMinSize((self.textctrl_width + 5 + (2*self.height), self.height+5))
	
	def __do_layout(self):
		# begin wxGlade: dir_picker.__do_layout
		sizer = wx.BoxSizer(wx.HORIZONTAL)
		sizer_5 = wx.BoxSizer(wx.VERTICAL)
		sizer_5.Add(self.dir_value, 1, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND, 0)
		sizer.Add(sizer_5, 1, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND, 0)
		sizer.Add(self.clear_btn, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		sizer.Add(self.browse_btn, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		self.SetSizer(sizer)
		sizer.Fit(self)
		self.Layout()
		# end wxGlade
	
	def set_textctrl_width(self, width):
		self.SetTextWidth(width)
	
	def set_height(self, height):
		self.SetHeight(height)
		
	def get_value(self):
		return self.GetValue()
	
	def reset_value(self):
		return self.ResetValue()
	
	def AppendText(self, text):
		"""
		Appends the text to the end of the text control.

		:param text: Text to write to the text control
		:type text: string
		"""
		return self.dir_value.AppendText(text)
	
	def Browse(self, *_):  # wxGlade: dir_picker.<event_handler>
		if self.get_value() == '':
			default_path = self.initial_value
		else:
			default_path = self.get_value()
		
		dlg = wx.DirDialog(
			None, "Choose a directory:",
			style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON,
			defaultPath=default_path
		)
		
		if dlg.ShowModal() == wx.ID_OK:
			self.dir_value.SetValue((dlg.GetPath()))
		dlg.Destroy()
		self.dir_value.SetFocus()
	
	def CanCopy(self):
		"""
		Returns True if the selection can be copied to the clipboard.

		:rtype: bool
		"""
		return self.dir_value.CanCopy()
	
	def CanCut(self):
		"""
		Returns True if the selection can be cut to the clipboard.

		:rtype: bool
		"""
		return self.dir_value.CanCut()
	
	def CanPaste(self):
		"""
		Returns True if the contents of the clipboard can be pasted into the text control.
		
		On some platforms (Motif, GTK) this is an approximation and returns True if the control is editable, False otherwise.

		:rtype: bool
		"""
		return self.dir_value.CanPaste()
	
	def CanRedo(self):
		"""
		Returns True if there is a redo facility available and the last operation can be redone.
		
		:rtype: bool
		"""
		return self.dir_value.CanRedo()

	def CanUndo(self):
		"""
		Returns True if there is an undo facility available and the last operation can be undone.
		
		:rtype:	bool
		"""
		return self.dir_value.CanUndo()

	def Clear(self, *_):  # wxGlade: dir_picker.<event_handler>
		"""
		Clears the text in the control.

		Note that this function will generate a wxEVT_TEXT event, i.e. its effect is identical to calling SetValue (“”).
		"""
		self.dir_value.Clear()
		self.dir_value.SetFocus()

	def Copy(self):
		"""
		Copies the selected text to the clipboard.
		"""
		return self.dir_value.Copy()

	def Cut(self):
		"""
		Copies the selected text to the clipboard and removes it from the control.
		"""
		return self.dir_value.Cut()
		
	def GetInsertionPoint(self):
		"""
		Returns the insertion point, or cursor, position.
	
		This is defined as the zero based index of the character position to the right of the insertion point. For example, if the insertion point is at the end of the single-line text control, it is equal to GetLastPosition .
	
		:rtype:	long
		"""
		return self.dir_value.GetInsertionPoint()

	def GetLastPosition(self):
		"""
		Returns the zero based index of the last position in the text control, which is equal to the number of characters in the control.
	
		:rtype:	wx.TextPos
		"""
		return self.dir_value.GetLastPosition()

	def GetSelection(self):
		"""
		Gets the current selection span.
	
		If the returned values are equal, there was no selection. Please note that the indices returned may be used with the other wx.TextCtrl methods but don’t necessarily represent the correct indices into the string returned by GetValue for multiline controls under Windows (at least,) you should use GetStringSelection to get the selected text.
	
		:rtype:	tuple
		"""
		return self.dir_value.GetSelection()

	def GetStringSelection(self):
		"""
		Gets the text currently selected in the control.
	
		If there is no selection, the returned string is empty.
	
		:rtype:	string
		"""
		return self.dir_value.GetStringSelection()

	def GetValue(self):
		"""
		Gets the contents of the control.
		
		Notice that for a multiline text control, the lines will be separated by (Unix-style) \n characters, even under Windows where they are separated by a \r\n sequence in the native control.
		
		:rtype:	string
		"""
		return self.dir_value.GetValue()
	
	def IsEditable(self):
		"""
		Returns True if the controls contents may be edited by user (note that it always can be changed by the program).
		
		In other words, this functions returns True if the control hasn’t been put in read-only mode by a previous call to SetEditable .
		
		:rtype:	bool
		"""
		return self.dir_value.IsEditable()

	def IsEmpty(self):
		"""
		Returns True if the control is currently empty.
		
		This is the same as GetValue .empty() but can be much more efficient for the multiline controls containing big amounts of text.
		
		:rtype:	bool
		"""
		return self.dir_value.IsEmpty()
	
	def Paste(self):
		"""
		Pastes text from the clipboard to the text item.
		"""
		return self.dir_value.Paste()

	def Redo(self):
		"""
		If there is a redo facility and the last operation can be redone, redoes the last operation.
		
		Does nothing if there is no redo facility.
		"""
		return self.dir_value.Redo()
		
	def Remove(self, from_, to_):
		"""
		Removes the text starting at the first given position up to (but not including) the character at the last position.
		
		This function puts the current insertion point position at to as a side effect.
		
		:param from_: The first position
		:type from_: long
		:param to_: The last position
		:type to_: long
		"""
		return self.dir_value.Remove(from_, to_)
	
	def Replace(self, from_, to_, value):
		"""
		Replaces the text starting at the first position up to (but not including) the character at the last position with the given text.
		
		This function puts the current insertion point position at to as a side effect.
		
		:param from_: The first position
		:type from_: long
		:param to_: The last position
		:type to_: long
		:param value: The value to replace the existing text with
		:type value: string
		"""
		return self.dir_value.Replace(from_, to_, value)

	def ResetValue(self):
		"""
		Resets the text ctrl to the initial value, either specified in __init__ or set with SetInitialValue
		"""
		return self.dir_value.SetValue(self.initial_value)

	def SelectAll(self):
		"""
		Selects all text in the control.
		
		See also SetSelection
		"""
		return self.dir_value.SelectAll()

	def SelectNone(self):
		"""
		Deselects selected text in the control.
		"""
		return self.dir_value.SelectNone()

	def SetHeight(self, height):
		"""
		Set the height of the widgets
		
		:param height: Height of the widgets
		:type height: float or int
		"""
		self.height = height
		self.__set_properties()
		self.Layout()

	def SetInitialValue(self, value):
		"""
		Sets the initial value for the text ctrl
		
		:param value: Initial value for the text ctrl
		:type value: string
		"""
		self.initial_value = value
		
	def SetInsertionPoint(self, pos):
		"""
		Sets the insertion point at the given position.
		
		:param pos: Position to set, in the range from 0 to GetLastPosition inclusive.
		:type pos: long
		:return:
		"""
		return self.dir_value.SetInsertionPoint(pos)

	def SetInsertionPointEnd(self):
		"""
		Sets the insertion point at the end of the text control.
		
		This is equivalent to calling wx.TextCtrl.SetInsertionPoint with wx.TextCtrl.GetLastPosition argument.
		"""
		return self.dir_value.SetInsertionPointEnd()

	def SetSelection(self, from_, to_):
		"""
		Selects the text starting at the first position up to (but not including) the character at the last position.
		
		If both parameters are equal to -1 all text in the control is selected.
		
		Notice that the insertion point will be moved to from by this function.
		
		:param from_: The first position
		:type from_: long
		:param to_: The last position
		:type to_: long
		
		See also SelectAll
		"""
		
		return self.dir_value.SetSelection(from_, to_)

	def SetTextWidth(self, width):
		"""
		Sets the width of the TextCtrl
		
		:param width: The width of the TextCtrl
		:type width: float or int
		"""
		
		self.textctrl_width = width
		self.__set_properties()
		self.Layout()

	def SetValue(self, value):
		"""
		Sets the new text control value.
		
		It also marks the control as not-modified which means that IsModified() would return False immediately after the call to SetValue .
		
		The insertion point is set to the start of the control (i.e. position 0) by this function unless the control value doesn’t change at all, in which case the insertion point is left at its original position.
		
		Note that, unlike most other functions changing the controls values, this function generates a wxEVT_TEXT event. To avoid this you can use ChangeValue instead.
		
		Parameters:	value (string) – The new value to set. It may contain newline characters if the text control is multi-line.
		"""
		
		return self.dir_value.SetValue(value)
		
	def Undo(self):
		"""
		If there is an undo facility and the last operation can be undone, undoes the last operation.
		
		Does nothing if there is no undo facility.
		"""
		
		return self.dir_value.Undo()

	def WriteText(self, text):
		"""
		Writes the text into the text control at the current insertion position.

		:param text: Text to write to the text control
		:type text: string
		"""
		
		return self.dir_value.WriteText(text)
	
# end of class dir_picker


class file_picker(dir_picker):
	"""

	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, value="", pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL | wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT, name=b"file_picker",
			extension="*", title="File Picker", filetypestring="All Files", **kwargs
	):
		"""

		:param parent:
		:type parent:
		:param id:
		:type id:
		:param value:
		:type value:
		:param pos:
		:type pos:
		:param size:
		:type size:
		:param style:
		:type style:
		:param name:
		:type name:
		:param extension:
		:type extension:
		:param title:
		:type title:
		:param filetypestring:
		:type filetypestring:
		:param kwargs:
		:type kwargs:
		"""
		
		self._parent = parent
		self.initial_value = value
		self.file_extension = extension
		self.dialog_title = title
		self.filetypestring = filetypestring
		self.file_dialog_kwargs = kwargs
		self.style = style
		
		dir_picker.__init__(self, parent, id, value, pos, size, style, name)
	
	def Browse(self, *_):  # wxGlade: dir_picker.<event_handler>
		if self.get_value() == '':
			default_path = str(pathlib.Path(self.initial_value).parent)
		else:
			default_path = pathlib.Path(self.get_value())
			if default_path.is_file():
				default_path = default_path.parent
			default_path = str(default_path)
		
		pathname = file_dialog(
			self, extension=self.file_extension,
			style=self.style,
			defaultDir=default_path,
			filetypestring=self.filetypestring,
			title=self.dialog_title,
		)
		
		if pathname:
			self.dir_value.SetValue(pathname)
			self.dir_value.SetFocus()

# end of class file_picker


class file_folder_picker(dir_picker):
	"""
	
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, value="", pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL | wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT, name=b"file_picker",
			extension="*", title="File Picker", filetypestring="All Files", start_as_files=True, **kwargs
	):
		"""

		:param parent:
		:type parent:
		:param id:
		:type id:
		:param value:
		:type value:
		:param pos:
		:type pos:
		:param size:
		:type size:
		:param style:
		:type style:
		:param name:
		:type name:
		:param extension:
		:type extension:
		:param title:
		:type title:
		:param filetypestring:
		:type filetypestring:
		:param kwargs:
		:type kwargs:
		"""
		
		self._parent = parent
		self.initial_value = value
		self.file_extension = extension
		self.dialog_title = title
		self.filetypestring = filetypestring
		self.file_dialog_kwargs = kwargs
		self.style = style
		self.files_mode = start_as_files
		
		dir_picker.__init__(self, parent, id, value, pos, size, style, name)
	
	def set_folders_mode(self):
		self.files_mode = False
	
	def set_files_mode(self):
		self.files_mode = True
	
	def toggle_mode(self):
		self.files_mode = not self.files_mode
		
	def Browse(self, *_):  # wxGlade: dir_picker.<event_handler>
		
		if self.files_mode:
		
			if self.get_value() == '':
				default_path = str(pathlib.Path(self.initial_value).parent)
			else:
				default_path = pathlib.Path(self.get_value())
				if default_path.is_file():
					default_path = default_path.parent
				default_path = str(default_path)
			
			pathname = file_dialog(
				self, extension=self.file_extension,
				style=self.style,
				defaultDir=default_path,
				filetypestring=self.filetypestring,
				title=self.dialog_title,
			)
			
			if pathname:
				self.dir_value.SetValue(pathname)
				self.dir_value.SetFocus()
	
		else:
			if self.get_value() == '':
				default_path = self.initial_value
			else:
				default_path = self.get_value()
			
			dlg = wx.DirDialog(
				None, "Choose a directory:",
				style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON,
				defaultPath=default_path
			)
			
			if dlg.ShowModal() == wx.ID_OK:
				self.dir_value.SetValue((dlg.GetPath()))
			dlg.Destroy()
			self.dir_value.SetFocus()
